package klaue.DownloadHelper;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.CookieHandler;
import java.net.HttpURLConnection;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.TreeMap;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

public class Downloader {
	public static void installCookieManager() {
		CookieHandler.setDefault(new ListCookieHandler());
	}
	
	/**
	 * Downloads a Webpage and returns the content
	 * Note: If the server times out, there will be retries until the download works.
	 * @param url The Webpage to download
	 * @param sReferrer The referrer. NULL for none
	 * @param sUserAgent The user agent to use. NULL for Opera 8.5, empty string for system default (java) 
	 * @param timeout The time to wait for the answer of the server in milliseconds
	 * @param retries The number of times to retry if error (negative for endless). Possible Errors for this: SocketTimeoutException, HTTP 504 - Gateway timeout, SocketException
	 * @return An ArrayList with the content of the page or null if error (and no retries left)
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public static ArrayList<String> downloadPage(URL url, String sReferrer, String sUserAgent, int timeout, int retries) throws FileNotFoundException, IOException {
		String sUrl = "";
		for (String splitted : url.toString().split("/")) {
			if (splitted.equals("http:") || splitted.equals("https:")) {
				sUrl += splitted + "/";
			} else {
				// you have no idea how complicated a simple url encoding can be...
				splitted = URLEncoder.encode(splitted, "UTF-8");
				
				// replace stuff that could be in a get request back in non-encoded form
				splitted = splitted.replace("%3F", "?").replace("%3D", "=").replace("%26", "&");
				
				// if a % is followed by 4 numbers, it was probably allready encoded
				int searchFrom = 0;
				int i = 0;
				while ((i = splitted.indexOf("%25", searchFrom)) != -1) {
					if (splitted.length() - i < 5) break;
					try {
						Integer.parseInt(splitted.substring(i + 1, i + 5), 16);
						// if we reach that, an allready encoded part was probably reencoded
						splitted = splitted.substring(0, i + 1) + splitted.substring(i + 3);
					} catch(NumberFormatException e) {
						// no changing
					}
					searchFrom = i + 1;
				}
				sUrl += splitted + "/";
			}
		}
		// remove last /
		if (!url.toString().substring(url.toString().length() - 1).equals("/")) sUrl = sUrl.substring(0, sUrl.length() - 1);
		url = new URL(sUrl);
		
		System.out.println(getTimestamp() + "Downloading page " + url.toString());
		ArrayList<String> page = new ArrayList<String>();
		
		URLConnection urlcon = url.openConnection();
		if (sUserAgent == null) {
			urlcon.setRequestProperty("User-Agent", "Mozilla/4.0(compatible;MSIE6.0;WindowsNT5.1;en)Opera8.54");
		} else {
			if (sUserAgent.length() != 0) urlcon.setRequestProperty("User-Agent", sUserAgent);
		}
		urlcon.setReadTimeout(timeout);
		if (sReferrer != null) {
			urlcon.setRequestProperty("Referer", sReferrer);
		} else {
			//reset referrer just in case it was set before
			urlcon.setRequestProperty("Referer", "");
		}
		
		
		BufferedReader URLinput = null;
		
		// get inputstream can throw a SocketTimeoutException
		try {
			urlcon.connect();
			URLinput = new BufferedReader(new InputStreamReader(urlcon.getInputStream(), "UTF8"));
			String sLine = "";
			while ((sLine = URLinput.readLine()) != null) {
				page.add(sLine);
			}
		} catch (SocketTimeoutException e) {
			System.err.print(getTimestamp() + e.getLocalizedMessage());
			// retry when timeout
			if (retries != 0) {
				System.out.println(" - Retry..");
    			try {
					Thread.sleep(1000);
				} catch (InterruptedException e1) {
					System.err.println("Couldn't sleep");
				}
				if (retries < 0) page = downloadPage(url, sReferrer, sUserAgent, timeout, -1);
				if (retries > 0) page = downloadPage(url, sReferrer, sUserAgent, timeout, (retries-1));
			} else {
				System.out.println();
				page = null;
			}
		} catch (SocketException e) {
			System.err.print(getTimestamp() + e.getLocalizedMessage());
			// most probably "unexpected end of file" or a ConnectException (Connection refused)
			// probably provoked by a proxy.
			// happens all the time for JAP when changing the service/stopping jap
			// retry
			if (retries != 0) {
				System.out.println(" - Retry..");
    			try {
					Thread.sleep(1000);
				} catch (InterruptedException e1) {
					System.err.println("Couldn't sleep");
				}
				if (retries < 0) page = downloadPage(url, sReferrer, sUserAgent, timeout, -1);
				if (retries > 0) page = downloadPage(url, sReferrer, sUserAgent, timeout, (retries-1));
			} else {
				System.out.println();
				page = null;
			}
		} catch (IOException e) {
			// try to retry if http error 504 (gateway timeout)
			if (e.getMessage().indexOf("Server returned HTTP response code: 504") == -1) {
				// rethrow IOErrors which are not 504
				throw e;
			}
			System.err.print(getTimestamp() + e.getLocalizedMessage());
			// 504 - Gateway timeout
			// retry - maybe the proxy answers again (proxy servers are prone to timeout)
			if (retries != 0) {
				System.out.println(" - Retry..");
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e1) {
					System.err.println("Couldn't sleep");
				}
				if (retries < 0) page = downloadPage(url, sReferrer, sUserAgent, timeout, -1);
				if (retries > 0) page = downloadPage(url, sReferrer, sUserAgent, timeout, (retries-1));
			} else {
				System.out.println();
				page = null;
			}
		} finally {
			if (URLinput != null) URLinput.close();
		}
		
		return page;
	}
	
	/**
	 * Downloads a binary file
	 * Note: If the server times out, there will be retries until the download works.
	 * @param filePath The URL to the file
	 * @param sSavePath The requested save path of the file
	 * @param sReferrer The referrer or NULL if none
	 * @param sUserAgent The user agent to use. NULL for Opera 8.5, empty string for system default (java) 
	 * @param timeout The time to wait for the answer of the server in milliseconds
	 * @param retries The number of times to retry if error (negative for endless). Possible Errors for this: SocketTimeoutException, HTTP 504 - Gateway timeout, SocketException
	 * @return A string to the actual downloaded file - can be different to the requested file path if special chars had to be removed - or null if error (and no retries left)
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	@SuppressWarnings("null")
	public static String downloadFile(URL filePath, String sSavePath, String sReferrer, String sUserAgent, int timeout, int retries) throws FileNotFoundException, IOException {
		String sUrl = "";
		for (String splitted : filePath.toString().split("/")) {
			if (splitted.equals("http:") || splitted.equals("https:")) {
				sUrl += splitted + "/";
			} else {
				splitted = URLEncoder.encode(splitted, "UTF-8");
				
				// replace stuff that could be in a get request back in non-encoded form
				splitted = splitted.replace("%3F", "?").replace("%3D", "=").replace("%26", "&");
				
				// if a % is followed by 4 numbers, it was probably allready encoded
				int searchFrom = 0;
				int i = 0;
				while ((i = splitted.indexOf("%25", searchFrom)) != -1) {
					if (splitted.length() - i < 5) break;
					try {
						Integer.parseInt(splitted.substring(i + 1, i + 5), 16);
						// if we reach that, an allready encoded part was probably reencoded
						splitted = splitted.substring(0, i + 1) + splitted.substring(i + 3);
					} catch(NumberFormatException e) {
						// no changing
					}
					searchFrom = i + 1;
				}
				sUrl += splitted + "/";
			}
		}
		// remove last /
		if (!filePath.toString().substring(filePath.toString().length() - 1).equals("/")) sUrl = sUrl.substring(0, sUrl.length() - 1);
		filePath = new URL(sUrl);
		
		//if (imgPath.toString().indexOf(" ") != -1) imgPath = new URL(imgPath.toString().replaceAll(" ", "%20"));
		System.out.println(getTimestamp() + "Downloading file " + filePath.toString());
		String sPath = sSavePath.substring(0, sSavePath.lastIndexOf(File.separator) + 1);
		String sFile = sSavePath.substring(sSavePath.lastIndexOf(File.separator) + 1);

		// replacing %FF stuff and forbidden chars
		int iPos = 0;
		while (sFile.indexOf("%", iPos) != -1) {
			iPos = sFile.indexOf("%");
			char c = (char)Integer.parseInt(sFile.substring(iPos + 1, iPos + 3), 16);
			sFile = sFile.substring(0, iPos) + c + sFile.substring(iPos + 3);
			iPos += 1; // + 1 and not + 3 because we replaced %xx with one single char
		}
		String sForbidden[] = new String[] {":", "?", "\"", "<", ">", "*", "|"};
		for (String sChar : sForbidden) {
			iPos = 0;
			while (sFile.indexOf(sChar, iPos) != -1) {
				iPos = sFile.indexOf(sChar);
				sFile = sFile.substring(0, iPos) + "_" + sFile.substring(iPos + 1);
			}
		}
		sSavePath = sPath + sFile;
		
		OutputStream out = new BufferedOutputStream(new FileOutputStream(sSavePath));
		URLConnection urlcon = filePath.openConnection();
		if (sUserAgent == null) {
			urlcon.setRequestProperty("User-Agent", "Mozilla/4.0(compatible;MSIE6.0;WindowsNT5.1;en)Opera8.54");
		} else {
			if (sUserAgent.length() != 0) urlcon.setRequestProperty("User-Agent", sUserAgent);
		}
		urlcon.setReadTimeout(timeout);
		if (sReferrer != null) {
			urlcon.setRequestProperty("Referer", sReferrer);
		} else {
			//reset referrer just in case it was set before
			urlcon.setRequestProperty("Referer", "");
		}
		InputStream in = null;
		
		try {
			urlcon.connect();
			in = urlcon.getInputStream();
			byte[] buffer = new byte[1024];
			int numRead = 0;
			while ((numRead = in.read(buffer)) != -1) {
				out.write(buffer, 0, numRead);
				out.flush();
			}
			return sSavePath;
		} catch (SocketTimeoutException e) {
			System.err.print(getTimestamp() + e.getLocalizedMessage());
			if (in != null) in.close();
			if (out != null) out.close();
			// retry when timeout
			if (retries != 0) {
				System.out.println(" - Retry..");
    			try {
					Thread.sleep(1000);
				} catch (InterruptedException e1) {
					System.err.println("Couldn't sleep");
				}
				if (retries < 0) return downloadFile(filePath, sSavePath, sReferrer, sUserAgent, timeout, -1);
				if (retries > 0) return downloadFile(filePath, sSavePath, sReferrer, sUserAgent, timeout, (retries-1));
			} else {
				System.out.println();
			}
			return null;
		} catch (SocketException e) {
			System.err.print(getTimestamp() + e.getLocalizedMessage());
			// most probably "unexpected end of file" or a ConnectException (Connection refused)
			// probably provoked by a proxy.
			// happens all the time for JAP when changing the service/stopping JAP
			if (in != null) in.close();
			if (out != null) out.close();
			// endles retry
			if (retries != 0) {
				System.out.println(" - Retry..");
    			try {
					Thread.sleep(1000);
				} catch (InterruptedException e1) {
					System.err.println("Couldn't sleep");
				}
				if (retries < 0) return downloadFile(filePath, sSavePath, sReferrer, sUserAgent, timeout, -1);
				if (retries > 0) return downloadFile(filePath, sSavePath, sReferrer, sUserAgent, timeout, (retries-1));
			} else {
				System.out.println();
			}
			return null;
		} catch (IOException e) {
			// try to retry if http error 504 (gateway timeout)
			if (e.getMessage().indexOf("Server returned HTTP response code: 504") == -1) {
				// rethrow IOErrors which are not 504
				throw e;
			}
			System.err.print(getTimestamp() + e.getLocalizedMessage());
			if (in != null) in.close();
			if (out != null) out.close();
			// 504 - Gateway timeout
			// retry - maybe the proxy answers again (proxy servers are prone to timeout)
			if (retries != 0) {
				System.out.println(" - Retry..");
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e1) {
					System.err.println("Couldn't sleep");
				}
				if (retries < 0) return downloadFile(filePath, sSavePath, sReferrer, sUserAgent, timeout, -1);
				if (retries > 0) return downloadFile(filePath, sSavePath, sReferrer, sUserAgent, timeout, (retries-1));
			} else {
				System.out.println();
			}
			return null;
		}
	}
	
	/**
	 * Sends parameters using POST to webpage and returns the resulting page.
	 * Note: If the server times out, there will be retries until the download works.
	 * @param url The URL to send the POST to
	 * @param sPost The POST parameters to send
	 * @param sReferrer The referrer or NULL for none
	 * @param sUserAgent The user agent to use. NULL for Opera 8.5, empty string for system default (java) 
	 * @param timeout The time to wait for the answer of the server in milliseconds
	 * @param retries The number of times to retry if error (negative for endless). Possible Errors for this: SocketTimeoutException, HTTP 504 - Gateway timeout, SocketException
	 * @return An ArrayList containing the answer of the web server or null if error (and no retries left)
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public static ArrayList<String> sendPOST(URL url, String sPost, String sReferrer, String sUserAgent, int timeout, int retries) throws FileNotFoundException, IOException {
		String sUrl = "";
		for (String splitted : url.toString().split("/")) {
			if (splitted.equals("http:") || splitted.equals("https:")) {
				sUrl += splitted + "/";
			} else {
				splitted = URLEncoder.encode(splitted, "UTF-8");
				
				// replace stuff that could be in a get request back in non-encoded form
				splitted = splitted.replace("%3F", "?").replace("%3D", "=").replace("%26", "&");
				
				// if a %25 is followed by 2 numbers, it was probably allready encoded
				int searchFrom = 0;
				int i = 0;
				while ((i = splitted.indexOf("%25", searchFrom)) != -1) {
					if (splitted.length() - i < 5) break;
					try {
						Integer.parseInt(splitted.substring(i + 1, i + 5), 16);
						// if we reach that, an allready encoded part was probably reencoded
						splitted = splitted.substring(0, i + 1) + splitted.substring(i + 3);
					} catch(NumberFormatException e) {
						// no changing
					}
					searchFrom = i + 1;
				}
				sUrl += splitted + "/";
			}
		}
		// remove last /
		if (!url.toString().substring(url.toString().length() - 1).equals("/")) sUrl = sUrl.substring(0, sUrl.length() - 1);
		url = new URL(sUrl);
		
		//sPost = URLEncoder.encode(sPost, "UTF-8");
		
		// if (url.toString().indexOf(" ") != -1) url = new URL(url.toString().replaceAll(" ", "%20"));
		sPost = sPost.replaceAll(" ", "%20");
		System.out.println(getTimestamp() + "Posting " + sPost + " on " + url.toString());
		
		ArrayList<String> answer = new ArrayList<String>();
		HttpURLConnection urlcon = (HttpURLConnection) url.openConnection();
		urlcon.setRequestMethod("POST");
		if (sUserAgent == null) {
			urlcon.setRequestProperty("User-Agent", "Mozilla/4.0(compatible;MSIE6.0;WindowsNT5.1;en)Opera8.54");
		} else {
			if (sUserAgent.length() != 0) urlcon.setRequestProperty("User-Agent", sUserAgent);
		}
    	urlcon.setReadTimeout(timeout);
		if (sReferrer != null) {
			urlcon.setRequestProperty("Referer", sReferrer);
		} else {
			//reset referrer just in case it was set before
			urlcon.setRequestProperty("Referer", "");
		}
		// Dem Server sagen was da kommt
    	urlcon.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
    	// DoOutput, damit wir POSTen k�nnen (also daten an den server schicken)
    	urlcon.setDoOutput(true);
    	
    	try {
    		urlcon.connect();
	    	DataOutputStream URLoutput = new DataOutputStream(urlcon.getOutputStream());
	    	// nun das wirkliche POSTen
	    	URLoutput.writeBytes(sPost);
	    	// ...und sicherstellen, dass alles geschickt wurde, bevor ich den stream schliesse
	    	URLoutput.flush();
	    	URLoutput.close();
	    	
	    	// die serverantwort abfangen
	    	// check if it's a 302 redirect to a different protocol (http -> https etc) and follow that manually
	    	if (urlcon.getResponseCode() == 302) {
	    		String redirectUrl = urlcon.getHeaderField("Location");
	    		String originalProt = sUrl.substring(0, sUrl.indexOf("://"));
	    		String redirectProt = redirectUrl.substring(0, redirectUrl.indexOf("://"));
	    		if (!originalProt.toLowerCase().equals(redirectProt.toLowerCase())) {
	    			// throw IOException to show user
	    			throw new IOException("Server returned HTTP response code: 302 \"redirected\" to different protocol, URL " + redirectUrl);
	    		}
	    	}
	    	BufferedReader URLinput = new BufferedReader(new InputStreamReader(urlcon.getInputStream(), "UTF8"));
	    	String sLine = "";
	        while ((sLine = URLinput.readLine()) != null) {
	        	answer.add(sLine);
	        }
	        URLinput.close();   // Schlie�en des Streams
    	} catch (SocketTimeoutException e) {
			System.err.print(getTimestamp() + e.getLocalizedMessage());
			// retry when timeout
    		if (retries != 0) {
    			System.out.println(" - Retry..");
    			try {
					Thread.sleep(1000);
				} catch (InterruptedException e1) {
					System.err.println("Couldn't sleep");
				}
				if (retries < 0) answer = sendPOST(url, sPost, sReferrer, sUserAgent, timeout, -1);
				if (retries > 0) answer = sendPOST(url, sPost, sReferrer, sUserAgent, timeout, (retries-1));
			} else {
				System.out.println();
				answer = null;
			}
		} catch (SocketException e) {
			System.err.print(getTimestamp() + e.getLocalizedMessage());
			// most probably "unexpected end of file" or a ConnectException (Connection refused)
			// probably provoked by a proxy.
			// happens all the time for JAP when changing the service/stopping JAP
			// endles retry
    		if (retries != 0) {
    			System.out.println(" - Retry..");
    			try {
					Thread.sleep(1000);
				} catch (InterruptedException e1) {
					System.err.println("Couldn't sleep");
				}
				if (retries < 0) answer = sendPOST(url, sPost, sReferrer, sUserAgent, timeout, -1);
				if (retries > 0) answer = sendPOST(url, sPost, sReferrer, sUserAgent, timeout, (retries-1));
			} else {
				System.out.println();
	    		answer = null;
			}
		} catch (IOException e) {
			// try to retry if http error 504 (gateway timeout)
			if (e.getMessage().indexOf("Server returned HTTP response code: 504") == -1) {
				// rethrow IOErrors which are not 504
				throw e;
			}
			System.err.print(getTimestamp() + e.getLocalizedMessage());
			// 504 - Gateway timeout
			// retry - maybe the proxy answers again (proxy servers are prone to timeout)
			if (retries != 0) {
				System.out.println(" - Retry..");
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e1) {
					System.err.println("Couldn't sleep");
				}
				if (retries < 0) answer = sendPOST(url, sPost, sReferrer, sUserAgent, timeout, -1);
				if (retries > 0) answer = sendPOST(url, sPost, sReferrer, sUserAgent, timeout, (retries-1));
			} else {
				System.out.println();
				answer = null;
			}
		}
		return answer;
	}
	
	/**
	 * Sends parameters using POST to webpage and returns the resulting page.
	 * Note: If the server times out, there will be retries until the download works.
	 * @param url The URL to send the POST to
	 * @param sPost the POST-parameters to send. Use the absolute file path for the file parameter value
	 * @param theFile The File to send
	 * @param sReferrer The referrer or NULL for none
	 * @return An ArrayList containing the answer of the web server or null if error (and no retries left)
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public static ArrayList<String> sendFileUsingPOST(URL url, TreeMap<String, String> mPost, File theFile, String sReferrer, String sUserAgent, int timeout, int retries) throws FileNotFoundException, IOException {
		if (!theFile.exists() || !theFile.isFile()) throw new FileNotFoundException();
		
		// check if treemap contains filename
		if (!mPost.values().contains(theFile.getAbsolutePath())) throw new IllegalArgumentException();
		
		String sLineEnd = "\r\n";
		String sTwoHyphens = "--";
		//String sBoundary =  "*****";
		String sBoundary = "---------------------------8872242159205931581522813857";
		
		String sUrl = "";
		for (String splitted : url.toString().split("/")) {
			if (splitted.equals("http:") || splitted.equals("https:")) {
				sUrl += splitted + "/";
			} else {
				splitted = URLEncoder.encode(splitted, "UTF-8");
				
				// replace stuff that could be in a get request back in non-encoded form
				splitted = splitted.replace("%3F", "?").replace("%3D", "=").replace("%26", "&");
				
				// if a %25 is followed by 2 numbers, it was probably allready encoded
				int searchFrom = 0;
				int i = 0;
				while ((i = splitted.indexOf("%25", searchFrom)) != -1) {
					if (splitted.length() - i < 5) break;
					try {
						Integer.parseInt(splitted.substring(i + 1, i + 5), 16);
						// if we reach that, an allready encoded part was probably reencoded
						splitted = splitted.substring(0, i + 1) + splitted.substring(i + 3);
					} catch(NumberFormatException e) {
						// no changing
					}
					searchFrom = i + 1;
				}
				sUrl += splitted + "/";
			}
		}
		// remove last /
		if (!url.toString().substring(url.toString().length() - 1).equals("/")) sUrl = sUrl.substring(0, sUrl.length() - 1);
		url = new URL(sUrl);
		
		//sPost = URLEncoder.encode(sPost, "UTF-8");
		
		// if (url.toString().indexOf(" ") != -1) url = new URL(url.toString().replaceAll(" ", "%20"));
		//sPost = sPost.replaceAll(" ", "%20");
		
		ArrayList<String> answer = new ArrayList<String>();
		//HttpURLConnection urlcon = (HttpURLConnection) url.openConnection();
		URLConnection urlcon = url.openConnection();
		//urlcon.setRequestMethod("POST");
		urlcon.setRequestProperty("Connection", "Keep-Alive");
		if (sUserAgent == null) {
			urlcon.setRequestProperty("User-Agent", "Mozilla/4.0(compatible;MSIE6.0;WindowsNT5.1;en)Opera8.54");
		} else {
			if (sUserAgent.length() != 0) urlcon.setRequestProperty("User-Agent", sUserAgent);
		}
    	urlcon.setReadTimeout(timeout);
		if (sReferrer != null) {
			urlcon.setRequestProperty("Referer", sReferrer);
		} else {
			//reset referrer just in case it was set before
			urlcon.setRequestProperty("Referer", "");
		}
		// Dem Server sagen was da kommt
		urlcon.setRequestProperty("Content-Type", "multipart/form-data;boundary=" + sBoundary);
    	// DoOutput, damit wir POSTen k�nnen (also daten an den server schicken)
		urlcon.setDoInput(true);
    	urlcon.setDoOutput(true);

		// Don't use a cached copy.
    	urlcon.setUseCaches(false);
    	
    	try {
    		//urlcon.connect();
	    	DataOutputStream URLoutput = new DataOutputStream(urlcon.getOutputStream());
	    	URLoutput.writeBytes(sTwoHyphens + sBoundary + sLineEnd);
	    	
	    	Iterator<String> iter = mPost.keySet().iterator();
	    	while (iter.hasNext()) {
	    		String sKey = iter.next();
	    		if (mPost.get(sKey).equals(theFile.getAbsolutePath())) {
	    			// file
	    			URLoutput.writeBytes("Content-Disposition: form-data; name=\"" + sKey + "\";"
	    					+ " filename=\"" + theFile.getName() +"\"" + sLineEnd);
	    			URLoutput.writeBytes("Content-Type: " + getContentType(theFile) + sLineEnd);
	    			URLoutput.writeBytes(sLineEnd);
	    			
	    			// create a buffer of maximum size
	    			FileInputStream fileInputStream = new FileInputStream(theFile);
	    			int bytesRead, bytesAvailable, bufferSize;
	    			int maxBufferSize = 1*1024*1024;
	    			bytesAvailable = fileInputStream.available();
	    			bufferSize = Math.min(bytesAvailable, maxBufferSize);
	    			byte[] buffer = new byte[bufferSize];

	    			// read file and write it into form...
	    			bytesRead = fileInputStream.read(buffer, 0, bufferSize);

	    			while (bytesRead > 0)
	    			{
	    				URLoutput.write(buffer, 0, bufferSize);
	    				bytesAvailable = fileInputStream.available();
	    				bufferSize = Math.min(bytesAvailable, maxBufferSize);
	    				bytesRead = fileInputStream.read(buffer, 0, bufferSize);
	    			}
	    			fileInputStream.close();
	    		} else {
	    			// normal data
	    			URLoutput.writeBytes("Content-Disposition: form-data; name=\"" + sKey + "\"" + sLineEnd);
	    			URLoutput.writeBytes(sLineEnd);
	    			URLoutput.writeBytes(mPost.get(sKey));
	    		}
	    		if (iter.hasNext()) {
		    		// send multipart form data necesssary after file data...
	    			URLoutput.writeBytes(sLineEnd + sTwoHyphens + sBoundary + sLineEnd);
	    			//URLoutput.writeBytes(sTwoHyphens + sBoundary + sTwoHyphens + sLineEnd);
	    		}
    			
	    		URLoutput.flush();
	    	}
    		// send multipart form data necesssary after file data...
			URLoutput.writeBytes(sLineEnd);
			URLoutput.writeBytes(sTwoHyphens + sBoundary + sTwoHyphens + sLineEnd);
			URLoutput.flush();
    		URLoutput.close();
	    	
	    	// die serverantwort abfangen
	    	BufferedReader URLinput = new BufferedReader(new InputStreamReader(urlcon.getInputStream(), "UTF8"));
	    	String sLine = "";
	        while ((sLine = URLinput.readLine()) != null) {
	        	answer.add(sLine);
	        }
	        URLinput.close();   // Schliessen des Streams
    	} catch (SocketTimeoutException e) {
			System.err.print(getTimestamp() + e.getLocalizedMessage());
			// retry when timeout
    		if (retries != 0) {
    			System.out.println(" - Retry..");
    			try {
					Thread.sleep(1000);
				} catch (InterruptedException e1) {
					System.err.println("Couldn't sleep");
				}
				if (retries < 0) answer = sendFileUsingPOST(url, mPost, theFile, sReferrer, sUserAgent, timeout, -1);
				if (retries > 0) answer = sendFileUsingPOST(url, mPost, theFile, sReferrer, sUserAgent, timeout, (retries-1));
			} else {
				System.out.println();
				answer = null;
			}
		} catch (SocketException e) {
			System.err.print(getTimestamp() + e.getLocalizedMessage());
			// most probably "unexpected end of file" or a ConnectException (Connection refused)
			// probably provoked by a proxy.
			// happens all the time for JAP when changing the service/stopping JAP
			// endles retry
    		if (retries != 0) {
    			System.out.println(" - Retry..");
    			try {
					Thread.sleep(1000);
				} catch (InterruptedException e1) {
					System.err.println("Couldn't sleep");
				}
				if (retries < 0) answer = sendFileUsingPOST(url, mPost, theFile, sReferrer, sUserAgent, timeout, -1);
				if (retries > 0) answer = sendFileUsingPOST(url, mPost, theFile, sReferrer, sUserAgent, timeout, (retries-1));
			} else {
				System.out.println();
	    		answer = null;
			}
		} catch (IOException e) {
			// try to retry if http error 504 (gateway timeout)
			if (e.getMessage().indexOf("Server returned HTTP response code: 504") == -1) {
				// rethrow IOErrors which are not 504
				throw e;
			}
			System.err.print(getTimestamp() + e.getLocalizedMessage());
			// 504 - Gateway timeout
			// retry - maybe the proxy answers again (proxy servers are prone to timeout)
			if (retries != 0) {
				System.out.println(" - Retry..");
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e1) {
					System.err.println("Couldn't sleep");
				}
				if (retries < 0) answer = sendFileUsingPOST(url, mPost, theFile, sReferrer, sUserAgent, timeout, -1);
				if (retries > 0) answer = sendFileUsingPOST(url, mPost, theFile, sReferrer, sUserAgent, timeout, (retries-1));
			} else {
				System.out.println();
				answer = null;
			}
		}
		return answer;
	}
	
	/**
	 * Disables all HTTPS Validation. In other words, makes the Java Virtual Machine trust every HTTPS certification without checking for
	 * validity. Only use if you do not care who issued the cert. USE WITH EXTREME CAUTION.
	 * Note: if you call this anywhere in your prog, it will be used until the prog terminates or you manually
	 * uninstall the trust manager
	 * 
	 * The secure way is using a keystore file. Get it this way:
	 * http://blogs.sun.com/andreas/entry/no_more_unable_to_find
	 * and add it using
	 * System.setProperty("javax.net.ssl.trustStore", "path-to-file");
	 * or using a TrustManager or what-do-I-know-use-your-imagination 
	 */
	public static void disableHTTPSCertificateValidation() {
		// Create a trust manager that does not validate certificate chains
	    TrustManager[] trustAllCerts = new TrustManager[]{
	        new X509TrustManager() {
	            public java.security.cert.X509Certificate[] getAcceptedIssuers() {
	                return null;
	            }
	            public void checkClientTrusted(
	                java.security.cert.X509Certificate[] certs, String authType) {
	            }
	            public void checkServerTrusted(
	                java.security.cert.X509Certificate[] certs, String authType) {
	            }
	        }
	    };
	    
	    // Install the all-trusting trust manager
	    try {
	        SSLContext sc = SSLContext.getInstance("SSL");
	        sc.init(null, trustAllCerts, new java.security.SecureRandom());
	        HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
	    } catch (Exception e) {
	    }
	}
	
	private static String getTimestamp() {
		return new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(new Date()) + ": ";
	}
	
	private static String getContentType(File f) {
		if (!f.exists()) return null;
		URLConnection urCon = null;
		try {
			URL ur = f.toURI().toURL();
			urCon = ur.openConnection();
		} catch (Exception ex) {
        	ex.printStackTrace();
            return null;
        }
		return urCon.getContentType();
	}
}
