package allrounder;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.TreeMap;
import java.util.TreeSet;

import javax.swing.JOptionPane;

import allrounder.logger.Logwriter;

public class DbHandler {
	private static DbHandler ref;
	Connection conn;
	
	private DbHandler() {
		Logwriter.getInstance().writeLine("creating database", false);
		StringLib lib = StringLib.getInstance();
		Statement stmt;
		// connect to db
		try {
	        Class.forName("org.hsqldb.jdbcDriver" );
			this.conn = DriverManager.getConnection("jdbc:hsqldb:file:" + DirHandler.getAppDataPath() + "/YiffIndexDB;shutdown=true", "sa", "");
			stmt = this.conn.createStatement();
		} catch (Exception e) {
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			JOptionPane.showMessageDialog(null, lib.getValue("DbHandler.connectionerror.text"), lib.getValue("DbHandler.connectionerror.title"), JOptionPane.ERROR_MESSAGE);
			System.exit(1);
			return;
		}
		
		// create tables if they don't exist
		// (if they exist, an sqlexception will be thrown)
		// if one table exists, all exist
		try {
			stmt.executeUpdate("CREATE TABLE Directories (Dir_Id INT IDENTITY, Path VARCHAR(250), Removmedia BOOLEAN)" );
			stmt.executeUpdate("CREATE CACHED TABLE Imagedata (Imagedata_Id INT IDENTITY, Hash VARCHAR(40), Headcount TINYINT, Yiff BOOLEAN, Width INT, Height INT, Changed BOOLEAN)" );
			stmt.executeUpdate("CREATE CACHED TABLE Image (Image_Id INT IDENTITY, Name VARCHAR(250), Dir_Id INT, Imagedata_Id INT, foreign key (Dir_Id) references Directories(Dir_Id), foreign key (Imagedata_Id) references Imagedata(Imagedata_Id))");
			
			stmt.executeUpdate("CREATE TABLE Gender (Gender_Id INT PRIMARY KEY)" );
			stmt.executeUpdate("CREATE CACHED TABLE GenderImagedata (Imagedata_ID INT, Gender_Id INT, foreign key (Imagedata_Id) references Imagedata(Imagedata_Id), foreign key(Gender_Id) references Gender(Gender_Id))" );
			stmt.executeUpdate("CREATE TABLE Spezies (Spezies_Id INT PRIMARY KEY)" );
			stmt.executeUpdate("CREATE CACHED TABLE SpeziesImagedata (Imagedata_Id INT, Spezies_Id INT, foreign key (Imagedata_Id) references Imagedata(Imagedata_Id), foreign key(Spezies_Id) references Spezies(Spezies_Id))" );
			stmt.executeUpdate("CREATE CACHED TABLE Artist (Artist_Id INT IDENTITY, Name VARCHAR(300))" );
			stmt.executeUpdate("CREATE CACHED TABLE ArtistImagedata (Imagedata_Id INT, Artist_Id INT, foreign key (Imagedata_Id) references Imagedata(Imagedata_Id), foreign key(Artist_Id) references Artist(Artist_Id))" );
			stmt.executeUpdate("CREATE TABLE Fetish (Fetish_Id INT PRIMARY KEY)" );
			stmt.executeUpdate("CREATE CACHED TABLE FetishImagedata (Imagedata_Id INT, Fetish_Id INT, foreign key (Imagedata_Id) references Imagedata(Imagedata_Id), foreign key(Fetish_Id) references Fetish(Fetish_Id))" );
			stmt.executeUpdate("CREATE CACHED TABLE Other (Other_Id INT IDENTITY, Text VARCHAR(300))" );
			stmt.executeUpdate("CREATE CACHED TABLE OtherImagedata (Imagedata_Id INT, Other_Id INT, foreign key (Imagedata_Id) references Imagedata(Imagedata_Id), foreign key(Other_Id) references Other(Other_Id))" );
			
			// general table resembling map
			stmt.executeUpdate("CREATE TABLE General (General_Id INT IDENTITY, Key VARCHAR(50), Value VARCHAR(250))" );
			
			// for plugins
			stmt.executeUpdate("CREATE TABLE Plugins (Plugin_Id INT IDENTITY, Path VARCHAR(250))" );
			
			// add firststart = true and the version
			stmt.executeUpdate("INSERT INTO General(General_Id, Key, Value) VALUES (NULL, 'Firststart', 'TRUE')");
			stmt.executeUpdate("INSERT INTO General(General_Id, Key, Value) VALUES (NULL, 'Version', '1.13')");
			
			// add genders, fetishes and spezies
			int i = 0;
			while (true) {
				try {
					lib.getValue("Tag.sex." + i + ".text");
					stmt.executeUpdate("INSERT INTO Gender(Gender_Id) VALUES (" + i + ")");
				} catch (IllegalArgumentException e) {
					// this WILL be thrown because i just iterate as long as there is no entry left
					break;
				}
				++i;
			}
			
			i = 0;
			while (true) {
				try {
					lib.getValue("fetishchooser.fetishes." + i + ".text");
					stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + i + ")");
				} catch (IllegalArgumentException e) {break;}
				++i;
			}
			
			String temp = lib.getValue(0 + ""); // 0=all spezies
			String[] speziesArr = temp.split(",");
			for (String s : speziesArr) {
				stmt.executeUpdate("INSERT INTO Spezies(Spezies_Id) VALUES (" + Integer.parseInt(s) + ")");
			}
		} catch (SQLException e) {
			boolean bugOfWrongVers104 = false;
			if (e.getMessage().indexOf("object name already exists") == -1) {
				e.printStackTrace();
				e.printStackTrace(Logwriter.getInstance().getPrintStream());
			} else {
				// DB allready existed, check version
				String DBVersion = getGeneral("Version");
				if (DBVersion.equals("1.0")) {
					// will not happen - DB was in another place in version 1.0
					DBVersion = "1.01";
				}
				if (DBVersion.equals("1.01")) {
					// update from 1.01
					// nothing to do, DB remained unchanged
					System.out.println("Updating DB to version 1.02");
					try {
						stmt.executeUpdate("UPDATE General set Value = 1.02 WHERE Key = 'Version'");
					} catch (SQLException e2) {
						e2.printStackTrace();
						Logwriter.getInstance().writeLine("Updating to 1.02 failed", true);
						e2.printStackTrace(Logwriter.getInstance().getPrintStream());
						return;
					}
					DBVersion = "1.02";
				}
				if (DBVersion.equals("1.02")) {
					// update from 1.02
					System.out.println("Updating DB to version 1.03");
					try {
						// adding new fetishes
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 76 + ")"); // hyper/huge tits
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 77 + ")"); // tiny penis
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 78 + ")"); // tiny tits
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 79 + ")"); // size difference
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 80 + ")"); // dominance
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 81 + ")"); // femdom
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 82 + ")"); // stink
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 83 + ")"); // birthing
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 84 + ")"); // Straightjacket
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 85 + ")"); // Breast expansion
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 86 + ")"); // Muscle growth
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 87 + ")"); // Egg planting
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 88 + ")"); // Underboobs
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 89 + ")"); // MPreg
						
						stmt.executeUpdate("UPDATE General set Value = 1.03 WHERE Key = 'Version'");
					} catch (SQLException e2) {
						e2.printStackTrace();
						Logwriter.getInstance().writeLine("Updating to 1.03 failed", true);
						e2.printStackTrace(Logwriter.getInstance().getPrintStream());
						return;
					}
					DBVersion = "1.03";
				}
				if (DBVersion.equals("1.03")) {
					// update from 1.03
					System.out.println("Updating DB to version 1.04");
					try {
						// note: because I fucked up an update, there may be some installations that are actually on 1.04 level
						// but with a version tag of 1.03, therefore the setting of the version is first
						stmt.executeUpdate("UPDATE General set Value = 1.04 WHERE Key = 'Version'");
						
						// adding new fetishes
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 90 + ")"); // creampie
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 91 + ")"); // 69
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 92 + ")"); // autofellatio
					} catch (SQLException e2) {
						e2.printStackTrace();
						Logwriter.getInstance().writeLine("Either updating to 1.04 failed or you previously had a damaged copy of 1.04", true);
						e2.printStackTrace(Logwriter.getInstance().getPrintStream());
					}
					DBVersion = "1.04";
				}
				if (DBVersion.equals("1.04")) {
					// update from 1.04
					System.out.println("Updating DB to version 1.05");
					try {
						// adding new fetishes
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 93 + ")"); // Vaginalsex

						stmt.executeUpdate("UPDATE General set Value = 1.05 WHERE Key = 'Version'");
					} catch (SQLException e2) {
						bugOfWrongVers104 = true; // bug fixed in 1.09
						e2.printStackTrace();
						Logwriter.getInstance().writeLine("Updating to 1.05 failed", true);
						e2.printStackTrace(Logwriter.getInstance().getPrintStream());
						//return;
					}
					DBVersion = "1.05";
				}
				if (DBVersion.equals("1.05")) {
					// update from 1.05
					System.out.println("Updating DB to version 1.06");
					try {
						// adding new fetishes
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 94 + ")"); // Nipple-licking
						
						stmt.executeUpdate("UPDATE General set Value = 1.06 WHERE Key = 'Version'");
					} catch (SQLException e2) {
						if (!bugOfWrongVers104) { // bug fixed in 1.09
							e2.printStackTrace();
							Logwriter.getInstance().writeLine("Updating to 1.06 failed", true);
							e2.printStackTrace(Logwriter.getInstance().getPrintStream());
							return;
						}
					}
					DBVersion = "1.06";
				}
				if (DBVersion.equals("1.06")) {
					// update from 1.06
					System.out.println("Updating DB to version 1.07");
					try {
						// adding new fetishes
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 95 + ")"); // x-ray-vision
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 96 + ")"); // tit grabbing/fondling
						
						stmt.executeUpdate("UPDATE General set Value = 1.07 WHERE Key = 'Version'");
					} catch (SQLException e2) {
						if (!bugOfWrongVers104) { // bug fixed in 1.09
							e2.printStackTrace();
							Logwriter.getInstance().writeLine("Updating to 1.07 failed", true);
							e2.printStackTrace(Logwriter.getInstance().getPrintStream());
							return;
						}
					}
					DBVersion = "1.07";
				}
				if (DBVersion.equals("1.07")) {
					// update from 1.07
					System.out.println("Updating DB to version 1.08");
					try {
						stmt.executeUpdate("UPDATE General set Value = 1.08 WHERE Key = 'Version'");
					} catch (SQLException e2) {
						e2.printStackTrace();
						Logwriter.getInstance().writeLine("Updating to 1.08 failed", true);
						e2.printStackTrace(Logwriter.getInstance().getPrintStream());
						return;
					}
					DBVersion = "1.08";
				}
				if (DBVersion.equals("1.08")) {
					// update from 1.08
					System.out.println("Updating DB to version 1.09");
					try {
						stmt.executeUpdate("UPDATE General set Value = 1.09 WHERE Key = 'Version'");
					} catch (SQLException e2) {
						e2.printStackTrace();
						Logwriter.getInstance().writeLine("Updating to 1.09 failed", true);
						e2.printStackTrace(Logwriter.getInstance().getPrintStream());
						return;
					}
					DBVersion = "1.09";
				}
				if (DBVersion.equals("1.09")) {
					// update from 1.09
					System.out.println("Updating DB to version 1.10");
					try {
						stmt.executeUpdate("UPDATE ImageData set Changed=true");
						stmt.executeUpdate("UPDATE General set Value = 1.10 WHERE Key = 'Version'");
					} catch (SQLException e2) {
						e2.printStackTrace();
						Logwriter.getInstance().writeLine("Updating to 1.10 failed", true);
						e2.printStackTrace(Logwriter.getInstance().getPrintStream());
						return;
					}
					DBVersion = "1.10";
				}
				if (DBVersion.equals("1.10")) {
					// update from 1.10
					System.out.println("Updating DB to version 1.11");
					try {
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + 97 + ")"); // bottomless
						stmt.executeUpdate("UPDATE General set Value = 1.11 WHERE Key = 'Version'");
					} catch (SQLException e2) {
						e2.printStackTrace();
						Logwriter.getInstance().writeLine("Updating to 1.11 failed", true);
						e2.printStackTrace(Logwriter.getInstance().getPrintStream());
						return;
					}
					DBVersion = "1.11";
				}
				if (DBVersion.equals("1.11")) {
					// update from 1.11
					System.out.println("Updating DB to version 1.12");
					try {
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (98), (99), (100), (101), (102), (103), (104), (105), (106), (107), " +
								"(108), (109), (110), (111), (112), (113), (114), (115), (116), (117), (118), (119), (120), (121), (122), (123), (124), " +
								"(125), (126), (127), (128), (129), (130), (131), (132), (133), (134), (135), (136), (137), (138), (139), (140)"); // diverse
						stmt.executeUpdate("INSERT INTO Spezies(Spezies_Id) VALUES (115), (116), (117)"); // dämon mod, alien, moogle
						stmt.executeUpdate("UPDATE General set Value = 1.12 WHERE Key = 'Version'");
					} catch (SQLException e2) {
						e2.printStackTrace();
						Logwriter.getInstance().writeLine("Updating to 1.12 failed", true);
						e2.printStackTrace(Logwriter.getInstance().getPrintStream());
						return;
					}
					DBVersion = "1.12";
				}
				if (DBVersion.equals("1.12")) {
					// update from 1.12 - nothing to do
					System.out.println("Updating DB to version 1.13");
					try {
						stmt.executeUpdate("UPDATE General set Value = 1.13 WHERE Key = 'Version'");
					} catch (SQLException e2) {
						e2.printStackTrace();
						Logwriter.getInstance().writeLine("Updating to 1.13 failed", true);
						e2.printStackTrace(Logwriter.getInstance().getPrintStream());
						return;
					}
					DBVersion = "1.13";
				}
			}
		}
		
	}
	
	public synchronized void destruct() {
		try {
			Logwriter.getInstance().writeLine("destructing database", false);
			Statement stmt = this.conn.createStatement();
			stmt.executeUpdate("SHUTDOWN");
			this.conn.close();
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
		this.conn = null;
	}
	
	public static synchronized DbHandler getInstance() {
		if (ref == null) {
			ref = new DbHandler();
		}
		return ref;
	}
	
	//just to be sure
	@Override
	public Object clone() throws CloneNotSupportedException {
	    throw new CloneNotSupportedException(); 
	}
	
	public void setGeneral(String Key, String Value) {
		try {
			Statement stmt = this.conn.createStatement();
			stmt.executeUpdate("UPDATE General SET Value = '" + Value + "' WHERE Key = '" + Key + "'");
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return;
		}
	}
	
	public String getGeneral(String Key) {
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Value FROM General WHERE Key = '" + Key + "'");
			if(!rs.next()) return "";
			
			return rs.getString("Value");
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return "";
		}
	}
	
	public ArrayList<String> getPugins() {
		ArrayList<String> alPlugins = new ArrayList<String>();
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Path FROM Plugins");
			
			while(rs.next()) {
				alPlugins.add(rs.getString("Path"));
			}
			return alPlugins;
			
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return alPlugins;
		}
	}
	
	public void addPlugin(String sClassPath) {
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Plugin_Id FROM Plugins WHERE Path = '" + sClassPath + "'");
			if (!rs.next()) stmt.executeUpdate("INSERT INTO Plugins(Plugin_Id,Path) VALUES (NULL,'" + sClassPath + "')");	
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
	}
	
	public void removePlugin(String sClassPath) {
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Plugin_Id FROM Plugins WHERE Path = '" + sClassPath + "'");
			if (rs.next()) stmt.executeUpdate("DELETE FROM Plugins WHERE Path = '" + sClassPath + "'");
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
	}
	
	public void setImageChanged(String sHash, boolean bChanged) {
		try {
			Statement stmt = this.conn.createStatement();
			stmt.executeUpdate("UPDATE Imagedata SET changed = " + Boolean.toString(bChanged) + " WHERE Hash = '" + sHash + "'");
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
	}
	
	// get all images
	public TreeSet<TaggedImage> getAllImages() {
		ArrayList<Integer> ids = new ArrayList<Integer>();
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Image_Id FROM Image");
			while(rs.next()) {
				ids.add(rs.getInt("Image_Id"));
			}
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		
		TreeSet<TaggedImage> imgs = new TreeSet<TaggedImage>();
		for (int id : ids) {
			imgs.add(getImageFromImageId(id));
		}
		
		return imgs;
	}
	
	public ArrayList<Integer> getAllImageIds() {
		ArrayList<Integer> ids = new ArrayList<Integer>();
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Image_Id FROM Image");
			while(rs.next()) {
				ids.add(rs.getInt("Image_Id"));
			}
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		
		return ids;
	}
	
	public ArrayList<Integer> getAllImageDataIds() {
		ArrayList<Integer> ids = new ArrayList<Integer>();
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Imagedata_Id FROM Imagedata");
			while(rs.next()) {
				ids.add(rs.getInt("Imagedata_Id"));
			}
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		
		return ids;
	}
	
	public ArrayList<File> getImagePathsOfDirectory(File dir) {
		ArrayList<File> imgs = new ArrayList<File>();
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rsDir = stmt.executeQuery("SELECT Dir_Id FROM Directories WHERE Path = '" + escapeQuotes(dir.getAbsolutePath()) + "'");
			if (!rsDir.next()) return imgs;
			
			int dirId = rsDir.getInt("Dir_Id");
			ResultSet rsImg = stmt.executeQuery("SELECT Name FROM Image WHERE Dir_Id = " + dirId);

			while (rsImg.next()) {
				imgs.add(new File(dir.getAbsolutePath() + File.separator + rsImg.getString("Name")));
			}
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		
		return imgs;
	}
	
	public ArrayList<TaggedImage> getImagesOfDirectory(String dir) {
		ArrayList<TaggedImage> imgs = new ArrayList<TaggedImage>();
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rsDir = stmt.executeQuery("SELECT Dir_Id FROM Directories WHERE Path = '" + escapeQuotes(dir) + "'");
			if (!rsDir.next()) return imgs;
			
			int dirId = rsDir.getInt("Dir_Id");
			ResultSet rsImg = stmt.executeQuery("SELECT Image_Id FROM Image WHERE Dir_Id = " + dirId);

			while (rsImg.next()) {
				imgs.add(getImageFromImageId(rsImg.getInt("Image_Id")));
			}
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		
		return imgs;
	}
	
	public TaggedImage getImageFromHash(String hash) {
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rsImagedata = stmt.executeQuery("SELECT Imagedata_Id FROM Imagedata WHERE Hash = '" + escapeQuotes(hash) + "'");
			if(!rsImagedata.next()) return null;
			
			int imgDataId = rsImagedata.getInt("Imagedata_Id");
			ResultSet rsImg = stmt.executeQuery("SELECT Image_Id FROM Image WHERE Imagedata_Id = " + imgDataId);
			if(!rsImg.next()) return null;
			return getImageFromImageId(rsImg.getInt("Image_Id"));
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
	}
	
	public File getImagePathFromId (int id) {
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Name,Dir_Id FROM Image WHERE Image_Id = " + id);
			if (!rs.next()) return null;
			String name = rs.getString("Name");
			rs = stmt.executeQuery("SELECT Path FROM Directories WHERE Dir_Id = " + rs.getInt("Dir_Id"));
			rs.next();
			return new File(rs.getString("Path") + File.separator + name);
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
	}
	
	public TaggedImage getImageFromImageId(int id) {
		TaggedImage img = new TaggedImage();
		img.ID = id;
		try {
			Statement stmt = this.conn.createStatement();
			
			// imagedataid and paths
			ResultSet rsImage = stmt.executeQuery("SELECT Name,Dir_Id,Imagedata_Id FROM Image WHERE Image_Id = " + id);
			if(!rsImage.next()) return null;
			int Dir_Id = rsImage.getInt("Dir_Id");
			ResultSet rsTmp = stmt.executeQuery("SELECT Path FROM Directories WHERE Dir_Id = " + Dir_Id);
			if (!rsTmp.next()) return null;
			img.path = new File(rsTmp.getString("Path") + File.separator + rsImage.getString("Name"));
			int imagedataId = rsImage.getInt("Imagedata_Id");
			
			// hash, headcount, yiff, width, height
			ResultSet rsImagedata = stmt.executeQuery("SELECT Hash,Headcount,Yiff,Width,Height,Changed FROM Imagedata WHERE Imagedata_Id = " + imagedataId);
			if(!rsImagedata.next()) return null;
			
			img.hash = rsImagedata.getString("Hash");
			img.headcount = rsImagedata.getInt("Headcount");
			img.yiff = rsImagedata.getBoolean("Yiff");
			img.width = rsImagedata.getInt("Width");
			img.height = rsImagedata.getInt("Height");
			img.changed = rsImagedata.getBoolean("Changed");
			
			// GenderImagedata
			rsTmp = stmt.executeQuery("SELECT Gender_Id FROM GenderImagedata WHERE Imagedata_Id = " + imagedataId);
			while (rsTmp.next()) {
				img.gender.add(rsTmp.getInt("Gender_Id"));
			}
			
			// SpeziesImage
			rsTmp = stmt.executeQuery("SELECT Spezies_Id FROM SpeziesImagedata WHERE Imagedata_Id = " + imagedataId);
			while (rsTmp.next()) {
				img.spezies.add(rsTmp.getInt("Spezies_Id"));
			}
			
			// ArtistImage
			rsTmp = stmt.executeQuery("SELECT Artist_Id FROM ArtistImagedata WHERE Imagedata_Id = " + imagedataId);
			while (rsTmp.next()) {
				int artId = rsTmp.getInt("Artist_Id");
				ResultSet rs = stmt.executeQuery("SELECT Name FROM Artist WHERE Artist_Id = " + artId);
				if (!rs.next()) continue; // there can be.. ONLY ONE!
				img.artists.add(rs.getString("Name"));
			}
			
			// OtherImage
			rsTmp = stmt.executeQuery("SELECT Other_Id FROM OtherImagedata WHERE Imagedata_Id = " + imagedataId);
			while (rsTmp.next()) {
				int otherId = rsTmp.getInt("Other_Id");
				ResultSet rs = stmt.executeQuery("SELECT Text FROM Other WHERE Other_Id = " + otherId);
				if (!rs.next()) continue; // there can be.. ONLY ONE!
				img.other.add(rs.getString("Text"));
			}
			
			// FetishImage
			rsTmp = stmt.executeQuery("SELECT Fetish_Id FROM FetishImagedata WHERE Imagedata_Id = " + imagedataId);
			while (rsTmp.next()) {
				img.fetish.add(rsTmp.getInt("Fetish_Id"));
			}
		} catch(SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		return img;
	}
	
	/**
	 * Gets TaggedImages from the DB using the imagedata-id. Be carfeul: the TaggedImages are incomplete
	 * and lack the image path and the image id - only use for export and online export
	 * @param imagedataId the imagedata id
	 * @return the incomplete Image as TaggedImage
	 */
	public TaggedImage getIncompleteImageFromImageDataId(int imagedataId) {
		TaggedImage img = new TaggedImage();
		img.ID = 0;
		try {
			Statement stmt = this.conn.createStatement();
			
			// hash, headcount, yiff, width, height
			ResultSet rsImagedata = stmt.executeQuery("SELECT Hash,Headcount,Yiff,Width,Height,Changed FROM Imagedata WHERE Imagedata_Id = " + imagedataId);
			if(!rsImagedata.next()) return null;
			
			img.hash = rsImagedata.getString("Hash");
			img.headcount = rsImagedata.getInt("Headcount");
			img.yiff = rsImagedata.getBoolean("Yiff");
			img.width = rsImagedata.getInt("Width");
			img.height = rsImagedata.getInt("Height");
			img.changed = rsImagedata.getBoolean("Changed");
			
			// GenderImagedata
			ResultSet rsTmp = stmt.executeQuery("SELECT Gender_Id FROM GenderImagedata WHERE Imagedata_Id = " + imagedataId);
			while (rsTmp.next()) {
				img.gender.add(rsTmp.getInt("Gender_Id"));
			}
			
			// SpeziesImage
			rsTmp = stmt.executeQuery("SELECT Spezies_Id FROM SpeziesImagedata WHERE Imagedata_Id = " + imagedataId);
			while (rsTmp.next()) {
				img.spezies.add(rsTmp.getInt("Spezies_Id"));
			}
			
			// ArtistImage
			rsTmp = stmt.executeQuery("SELECT Artist_Id FROM ArtistImagedata WHERE Imagedata_Id = " + imagedataId);
			while (rsTmp.next()) {
				int artId = rsTmp.getInt("Artist_Id");
				ResultSet rs = stmt.executeQuery("SELECT Name FROM Artist WHERE Artist_Id = " + artId);
				if (!rs.next()) continue; // there can be.. ONLY ONE!
				img.artists.add(rs.getString("Name"));
			}
			
			// OtherImage
			rsTmp = stmt.executeQuery("SELECT Other_Id FROM OtherImagedata WHERE Imagedata_Id = " + imagedataId);
			while (rsTmp.next()) {
				int otherId = rsTmp.getInt("Other_Id");
				ResultSet rs = stmt.executeQuery("SELECT Text FROM Other WHERE Other_Id = " + otherId);
				if (!rs.next()) continue; // there can be.. ONLY ONE!
				img.other.add(rs.getString("Text"));
			}
			
			// FetishImage
			rsTmp = stmt.executeQuery("SELECT Fetish_Id FROM FetishImagedata WHERE Imagedata_Id = " + imagedataId);
			while (rsTmp.next()) {
				img.fetish.add(rsTmp.getInt("Fetish_Id"));
			}
		} catch(SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		return img;
	}
	
	public int getImageIdFromPath (File path) {
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Image.Image_Id " +
					"FROM Image, Directories " +
					"WHERE Image.Dir_Id=Directories.Dir_Id AND " +
					"Image.Name = '" + escapeQuotes(path.getName()) + "' AND " +
					"Directories.Path = '" + escapeQuotes(path.getParent()) + "'");
			if (!rs.next()) return -1;
			return rs.getInt("Image_Id");
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
		return -1;
	}
	
	public TaggedImage getImageFromPath (File path) {
		int id = getImageIdFromPath(path);
		if (id == -1) return new TaggedImage();
		return getImageFromImageId(id);
	}
	
	public void changePathOfImgInDb(int ImgId, File path) {
		String folder = path.getParent();
		
		try {
			Statement stmt = this.conn.createStatement();
			
			// get folder-id from db
			ResultSet rsDir = stmt.executeQuery("SELECT Dir_Id FROM Directories WHERE Path = '" + escapeQuotes(folder) + "'");
			rsDir.next();
			int dirId = rsDir.getInt("Dir_Id");
			
			//delete every entry of the same pic
			//stmt.executeUpdate("DELETE FROM Image WHERE Dir_Id = " + dirId + " AND Name = '" + escapeQuotes(path.getName()) + "'");
			
			//update folder/image
			stmt.executeUpdate("UPDATE Image SET Name = '" + escapeQuotes(path.getName()) + "', Dir_Id = " + dirId + " WHERE Image_Id = " + ImgId);
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
	}
	
	private String escapeQuotes(String string) {
		int actualindex = 0;
		while (string.indexOf('\'', actualindex) != -1) {
			actualindex = string.indexOf('\'', actualindex);
			string = string.substring(0, actualindex) + "''" + string.substring(actualindex + 1);
			actualindex += 2; // 2 quotes
		}
		return string;
	}
	
	public TreeMap<File, Boolean> getDirectories() {
		try {
			Statement stmt = this.conn.createStatement();
			
			TreeMap<File, Boolean> dirs = new TreeMap<File, Boolean>();
			ResultSet rs = stmt.executeQuery("SELECT Path, Removmedia FROM Directories");
			while(rs.next()) {
				dirs.put(new File(rs.getString("Path")), rs.getBoolean("Removmedia"));
			}
			return dirs;
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
		return null;
	}
	
	public void addFolderToDatabase(File folder, Boolean RemovMedia) {
		try {
			Statement stmt = this.conn.createStatement();
			
			ResultSet rs = stmt.executeQuery("SELECT Dir_Id, Removmedia FROM Directories WHERE Path = '" + escapeQuotes(folder.getAbsolutePath()) + "'");
			if (rs.next()) {
				if (rs.getBoolean("Removmedia") != RemovMedia) {
					stmt.executeUpdate("UPDATE Directories SET Removmedia = " + RemovMedia.booleanValue() + " WHERE Dir_Id = " + rs.getInt("Dir_Id"));
				}
				return; // do not add a folder which allready exists
			}
			// NULL in a IDENTIFY-Field = use autogenerated number
			stmt.executeUpdate("INSERT INTO Directories(Dir_Id,Path,Removmedia) VALUES (NULL,'" + escapeQuotes(folder.getAbsolutePath()) + "', " + RemovMedia.booleanValue() + ")");
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
	}
	
	public void removeFolderFromDatabase(File folder) {
		try {
			Statement stmt = this.conn.createStatement();
			
			ResultSet rs = stmt.executeQuery("SELECT Dir_Id FROM Directories WHERE Path = '" + escapeQuotes(folder.getAbsolutePath()) + "'");
			if (rs.next()) {
				stmt.executeUpdate("DELETE FROM Directories WHERE Dir_Id = " + rs.getInt("Dir_Id"));
			}
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
	}
	
	public boolean isImageInDb(File path) {
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Image.Image_Id " +
					"FROM Image, Directories " +
					"WHERE Image.Dir_Id=Directories.Dir_Id AND " +
					"Image.Name = '" + escapeQuotes(path.getName()) + "' AND " +
					"Directories.Path = '" + escapeQuotes(path.getParent()) + "'");
			boolean returnVal = rs.next();
			stmt.close();
			return returnVal;
		} catch (SQLException e) {
			e.printStackTrace(); //ResultSet rs = stmt.executeQuery("SELECT Name,Dir_Id FROM Image WHERE Image_Id = " + id); ??????
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
		return false;
	}
	
	public ArrayList<Boolean> areImagesInDb(ArrayList<File> paths) {
		// make an sql of 20 at a time
		ArrayList<Boolean> answer = new ArrayList<Boolean>(paths.size());
			
		try {
			// get all images in db
			Statement stmt = this.conn.createStatement();
			HashSet<File> dbSet = new HashSet<File>();
			ResultSet rs = stmt.executeQuery("SELECT Directories.Path, Image.Name FROM Image, Directories WHERE Image.Dir_Id=Directories.Dir_Id");
			while (rs.next()) {
				dbSet.add(new File(rs.getString(1), rs.getString(2)));
			}
			stmt.close();
			
			// look if the images are on the db
			for (int i = 0; i < paths.size(); ++i) {
				File f = paths.get(i);
				answer.add(dbSet.contains(f));
			}
			
			dbSet = null;
			System.gc();
		} catch (SQLException e) {
			e.printStackTrace(); //ResultSet rs = stmt.executeQuery("SELECT Name,Dir_Id FROM Image WHERE Image_Id = " + id); ??????
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
		return answer;
	}
	
	/**
	 * Gets the ID of the artist (adds the artist to the DB if not there allready)
	 * @param artist The artist
	 * @return the ID of the allready existing or new artist
	 */
	public Integer getArtistId(String artist) {
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Artist_Id FROM Artist WHERE lower(Name) = lower('" + escapeQuotes(artist) + "')");
			if(rs.next()) return rs.getInt("Artist_Id");
			stmt.executeUpdate("INSERT INTO Artist(Artist_Id,Name) VALUES (NULL, '" + escapeQuotes(artist) + "')");
			rs = stmt.executeQuery("CALL IDENTITY();");
			rs.next();
			return rs.getInt(1);
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
		return null;
	}
	
	public ArrayList<String> getArtists() {
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Name FROM Artist");
			ArrayList<String> returnVal = new ArrayList<String>();
			while(rs.next()){
				returnVal.add(rs.getString("Name"));
			}
			return returnVal;
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
		return null;
	}
	
	public ArrayList<Integer> getImagedataIdsByArtist(String sArtist) {
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Artist_Id FROM Artist WHERE lower(Name) = lower('" + escapeQuotes(sArtist) + "')");
			if(!rs.next()) return null;
			int artistid = rs.getInt("Artist_Id");
			
			rs = stmt.executeQuery("SELECT Imagedata_Id FROM ArtistImagedata WHERE Artist_Id = " + artistid);
			ArrayList<Integer> returnVal = new ArrayList<Integer>();
			while(rs.next()){
				returnVal.add(rs.getInt("Imagedata_Id"));
			}
			return returnVal;
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
		return null;
	}
	
	public Integer getOtherId(String other) {
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Other_Id FROM Other WHERE lower(Text) = lower('" + escapeQuotes(other) + "')");
			if(rs.next()) return rs.getInt("Other_Id");
			stmt.executeUpdate("INSERT INTO Other(Other_Id,Text) VALUES (NULL, '" + escapeQuotes(other) + "')");
			rs = stmt.executeQuery("CALL IDENTITY();");
			rs.next();
			return rs.getInt(1);
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
		return null;
	}
	
	public ArrayList<String> getOthers() {
		try {
			Statement stmt = this.conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT Text FROM Other");
			ArrayList<String> returnVal = new ArrayList<String>();
			while(rs.next()){
				returnVal.add(rs.getString("Text"));
			}
			return returnVal;
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
		return null;
	}
	
	public void addImageToDatabase(TaggedImage img) {
		try {
			Statement stmt = this.conn.createStatement();
			// if image is allready in DB, remove previous tags
			if (isImageInDb(img.path)) {
				removeImageFromDatabase(img.path);
			}
			
			//check if the hash from this image is allready in the DB
			ResultSet rs = stmt.executeQuery("SELECT Imagedata_Id FROM Imagedata WHERE Hash = '" + img.hash + "'");
			int imgdataId = 0;
			if (!rs.next()) {
				// if no such imagedata exists
				// insert stuff into Imagedata-Table
				stmt.executeUpdate("INSERT INTO Imagedata(Imagedata_Id,Hash,Headcount,Yiff,Width,Height,Changed) VALUES (NULL, '" + escapeQuotes(img.hash) + "', " + img.headcount + ", " + img.yiff + ", " + img.width + ", " + img.height + ", " + img.changed + ")");
				rs = stmt.executeQuery("CALL IDENTITY();");
				rs.next();
				imgdataId = rs.getInt(1);
			} else {
				imgdataId = rs.getInt("Imagedata_Id");
				//change this imagedata

				stmt.executeUpdate("UPDATE Imagedata SET Headcount = " + img.headcount + ", Yiff = " + img.yiff + ", Changed = " + img.changed + " WHERE Imagedata_Id = " + imgdataId);
				stmt.executeUpdate("DELETE FROM GenderImagedata WHERE Imagedata_Id = " + imgdataId);
				stmt.executeUpdate("DELETE FROM SpeziesImagedata WHERE Imagedata_Id = " + imgdataId);
				// remove ArtistImage
				removeAllArtistsFromImagedata(imgdataId);
				
				// remove OtherImage
				rs = stmt.executeQuery("SELECT Other_Id FROM OtherImagedata WHERE Imagedata_Id = " + imgdataId);
				ArrayList<Integer> others = new ArrayList<Integer>();
				while(rs.next()) {
					others.add(rs.getInt("Other_Id"));
				}
				stmt.executeUpdate("DELETE FROM OtherImagedata WHERE Imagedata_Id = " + imgdataId);
				for (int i : others) {
					rs = stmt.executeQuery("SELECT Imagedata_Id FROM OtherImagedata WHERE Other_Id = " + i);
					if (!rs.next()) {
						//the deleted image was the only one
						stmt.executeUpdate("DELETE FROM Other WHERE Other_Id = " + i);
					}
				}
			}
			
			// GenderImagedata
			for (int id : img.gender) {
				stmt.executeUpdate("INSERT INTO GenderImagedata(Imagedata_Id,Gender_Id) VALUES (" + imgdataId + ", " + id + ")");
			}
			
			// SpeziesImage
			for (int id : img.spezies) {
				rs = stmt.executeQuery("SELECT Spezies_Id FROM Spezies WHERE Spezies_Id = " + id);
				if (!rs.next()) {
					// spezies is not in database -> was added after first run of YiffIndex
					stmt.executeUpdate("INSERT INTO Spezies(Spezies_Id) VALUES (" + id + ")");
				}

				stmt.executeUpdate("INSERT INTO SpeziesImagedata(Imagedata_Id,Spezies_Id) VALUES (" + imgdataId + ", " + id + ")");
			}
			
			// ArtistImage
			addArtistsToImagedata(img.artists, imgdataId);
			
			// FetishImage
			if (img.yiff) {
				for (int id : img.fetish) {
					rs = stmt.executeQuery("SELECT Fetish_Id FROM Fetish WHERE Fetish_Id = " + id);
					if (!rs.next()) {
						// fetish is not in database -> was added after first run of YiffIndex
						stmt.executeUpdate("INSERT INTO Fetish(Fetish_Id) VALUES (" + id + ")");
					}
					stmt.executeUpdate("INSERT INTO FetishImagedata(Imagedata_Id,Fetish_Id) VALUES (" + imgdataId + ", " + id + ")");
				}
			}
			
			// OtherImage
			for (String other : img.other) {
				int id = getOtherId(other);
				stmt.executeUpdate("INSERT INTO OtherImagedata(Imagedata_Id,Other_Id) VALUES (" + imgdataId + ", " + id + ")");
			}
			
			
			
			// get id of directory
			String directory = img.path.getParent();
			rs = stmt.executeQuery("SELECT Dir_Id FROM Directories WHERE Path = '" + escapeQuotes(directory) + "'");
			rs.next(); // no check because directory exists under all circumstances
			int dirId = rs.getInt("Dir_Id");
			stmt.executeUpdate("INSERT INTO Image(Image_Id,Name,Dir_Id,Imagedata_Id) VALUES (NULL, '" + escapeQuotes(img.path.getName()) + "', " + dirId + ", " + imgdataId + ")");
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
	}
	
	public void removeImageFromDatabase(File path) {
		try {
			Statement stmt = this.conn.createStatement();
			// get id of image, directory
			ResultSet rs = stmt.executeQuery("SELECT Image.Image_Id FROM Image, Directories WHERE Image.Dir_Id=Directories.Dir_Id AND Image.Name = '" + escapeQuotes(path.getName()) + "' AND Directories.Path = '" + escapeQuotes(path.getParent()) + "'");
			if (!rs.next()) return; // image is not in db
			int imgId = rs.getInt("Image_Id");
			removeImageFromDatabase(imgId);
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
	}
	
	public void removeImageFromDatabase(int imgId) {
		try {
			Statement stmt = this.conn.createStatement();
			
			// test if there is more than one image with the same imagedataid
			ResultSet rs = stmt.executeQuery("SELECT Imagedata_Id FROM Image WHERE Image_Id = " + imgId);
			if (!rs.next()) return; // there is none
			int imgdataId = rs.getInt("Imagedata_Id");
			rs = stmt.executeQuery("SELECT Image_Id FROM Image WHERE Imagedata_Id = " + imgdataId);
			rs.next();
			if (rs.next()) {
				// there is a minimum of two images with this imagedata, so we can just remove the image
				stmt.executeUpdate("DELETE FROM Image WHERE Image_Id = " + imgId);
				return;
			}
			
			//if we got to here, it's the only image with this imgdataId, so we have to destroy the imagedata
			
			// remove GenderImagedata
			stmt.executeUpdate("DELETE FROM GenderImagedata WHERE Imagedata_Id = " + imgdataId);
			
			// remove FetishImage
			stmt.executeUpdate("DELETE FROM FetishImagedata WHERE Imagedata_Id = " + imgdataId);
			
			// remove SpeziesImage
			stmt.executeUpdate("DELETE FROM SpeziesImagedata WHERE Imagedata_Id = " + imgdataId);
			
			// remove ArtistImage
			rs = stmt.executeQuery("SELECT Artist_Id FROM ArtistImagedata WHERE Imagedata_Id = " + imgdataId);
			ArrayList<Integer> artists = new ArrayList<Integer>();
			while(rs.next()) {
				artists.add(rs.getInt("Artist_Id"));
			}
			stmt.executeUpdate("DELETE FROM ArtistImagedata WHERE Imagedata_Id = " + imgdataId);
			for (int i : artists) {
				rs = stmt.executeQuery("SELECT Imagedata_Id FROM ArtistImagedata WHERE Artist_Id = " + i);
				if (!rs.next()) {
					//the deleted image was the only one
					stmt.executeUpdate("DELETE FROM Artist WHERE Artist_Id = " + i);
				}
			}
			
			// remove OtherImage
			rs = stmt.executeQuery("SELECT Other_Id FROM OtherImagedata WHERE Imagedata_Id = " + imgdataId);
			ArrayList<Integer> others = new ArrayList<Integer>();
			while(rs.next()) {
				others.add(rs.getInt("Other_Id"));
			}
			stmt.executeUpdate("DELETE FROM OtherImagedata WHERE Imagedata_Id = " + imgdataId);
			for (int i : others) {
				rs = stmt.executeQuery("SELECT Imagedata_Id FROM OtherImagedata WHERE Other_Id = " + i);
				if (!rs.next()) {
					//the deleted image was the only one
					stmt.executeUpdate("DELETE FROM Other WHERE Other_Id = " + i);
				}
			}
			
			//remove image
			stmt.executeUpdate("DELETE FROM Image WHERE Image_Id = " + imgId);
			
			// remove imagedata
			stmt.executeUpdate("DELETE FROM Imagedata WHERE Imagedata_Id = " + imgdataId);
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
	}
	
	public void removeAllArtistsFromImagedata(int imgdataId) {
		try {
			Statement stmt = this.conn.createStatement();
			
			ResultSet rs = stmt.executeQuery("SELECT Artist_Id FROM ArtistImagedata WHERE Imagedata_Id = " + imgdataId);
			ArrayList<Integer> artists = new ArrayList<Integer>();
			while(rs.next()) {
				artists.add(rs.getInt("Artist_Id"));
			}
			stmt.executeUpdate("DELETE FROM ArtistImagedata WHERE Imagedata_Id = " + imgdataId);
			for (int i : artists) {
				rs = stmt.executeQuery("SELECT Imagedata_Id FROM ArtistImagedata WHERE Artist_Id = " + i);
				if (!rs.next()) {
					//the deleted artist was the only one
					stmt.executeUpdate("DELETE FROM Artist WHERE Artist_Id = " + i);
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
	}
	
	public void addArtistsToImagedata(TreeSet<String> artists, int imgdataId) {
		try {
			Statement stmt = this.conn.createStatement();
			for (String artist : artists) {
				int id = getArtistId(artist);
				stmt.executeUpdate("INSERT INTO ArtistImagedata(Imagedata_Id,Artist_Id) VALUES (" + imgdataId + ", " + id + ")");
			}
		} catch (SQLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
	}
}


/*
	Klaue's little SQL cheat sheet
	### correcting db after adding new spezies/fetish
	# 4 = differenze between old and new id, 100 = lowest affected id
	update fetishimagedata set fetish_id = fetish_id + 4 where fetish_id >= 100
	update speziesimagedata set spezies_id = spezies_id + 4 where spezies_id >= 100
	
	### Adding a species to a character (other-tag):
	# 100 is an example spezies id
	# use only if sure that the spezies was not set to a single pic with Character allready
	INSERT INTO speziesimagedata (IMAGEDATA_ID, SPEZIES_ID)
	SELECT IMAGEDATA_ID, 100 as SPEZIES_ID
	FROM   otherimagedata
	WHERE  other_id = (
		select other_id from other where text = 'Charactername'
	)
	
	### Adding additional artist name to artist (second nickname)
	# use only if sure that the nick was not set to a single pic with old artist name allready
	# note: untested, use with care
	Insert into artist (name) values ('New nick')
	
	Select artist_id from artist where name = 'New nick' # assuming it to be 100 below
	
	INSERT INTO artistimagedata (IMAGEDATA_ID, ARTIST_ID)
	SELECT IMAGEDATA_ID, 100 as ARTIST_ID
	FROM   artistimagedata
	WHERE  ARTIST_ID = (
		select ARTIST_ID from other where text = 'Old nick'
	)
*/