package allrounder;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Locale;
import java.util.TreeMap;

import javax.swing.JOptionPane;

import allrounder.logger.Logwriter;

public class StringLib {
	private static StringLib ref;
	private File fDirectoryPath = new File(DirHandler.getAppDir(), "languages");
	private TreeMap<String, String> mStrings = null;

	private StringLib() {
		try {
			this.mStrings = getLanguageMapBySystemLocale(this.fDirectoryPath, Locale.ENGLISH);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		if (this.mStrings == null) {
			//abort
			JOptionPane.showMessageDialog(null, "Language file not found or not valid " + this.fDirectoryPath.getAbsolutePath(), "ERROR", JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
	}

	public static synchronized StringLib getInstance() {
		if (ref == null) {
			ref = new StringLib();
		}
		return ref;
	}
	
	//just to be sure
	@Override
	public Object clone() throws CloneNotSupportedException {
	    throw new CloneNotSupportedException(); 
	}
	
	public static TreeMap<String, String> getLanguageMapBySystemLocale(File directory, Locale defaultLanguage) {
		File fLanguageFile = new File(directory.getAbsolutePath() + File.separator + Locale.getDefault().getLanguage() + ".lng");
		
		if (!fLanguageFile.exists()) {
			// default to defaultLanguage
			fLanguageFile = new File(directory.getAbsolutePath() + File.separator + defaultLanguage.getLanguage() + ".lng");
			
			if (!fLanguageFile.exists()) {
				//abort
				//System.err.println("Language file not found");
				return null;
			}
		}
		
		TreeMap<String, String> langMap = new TreeMap<String, String>();
		try {
			BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(fLanguageFile), "UTF8"));
			//BufferedReader in = new BufferedReader(new FileReader(vLangFiles.elementAt(i)));
	        String str;
	        while ((str = in.readLine()) != null) {
	        	str = str.trim();
	        	if (str.startsWith("#")) continue;
	        	if (str.indexOf("=") == -1) continue;
	            String sName = str.substring(0, str.indexOf("=", 0)).trim();
	            String sValue = str.substring(str.indexOf("=", 0) + 1, str.length()).trim();
	            langMap.put(sName, sValue);
	        }
	        in.close();
	    } catch (IOException e) {
	    	e.printStackTrace();
	    	e.printStackTrace(Logwriter.getInstance().getPrintStream());
	    	return null;
	    }
	    return langMap;
	}
	
	public String getValue(String sName) throws IllegalArgumentException {
		if (this.mStrings == null) return "LANG_ERR";
		String sReturn = this.mStrings.get(sName);
		if (sReturn == null) {
			throw new IllegalArgumentException("ID \"" + sName + "\" not found");
		}
		if (sReturn.indexOf("\\n") != -1) {
			sReturn = "<html><body>" + sReturn.replaceAll("\\\\n", "<br>") + "</body></html>";
        }
		return sReturn;
	}
	
	// to add someting to the start/end of html-strings
	public String getValue(String sName, String sStart, String sEnd) throws IllegalArgumentException {
		if (sStart == null) sStart = "";
		if (sEnd == null) sEnd = "";
		String sReturn = this.mStrings.get(sName);
		if (sReturn == null) {
			throw new IllegalArgumentException("ID \"" + sName + "\" not found");
		}
		if (sReturn.indexOf("\\n") != -1) {
			sReturn = "<html><body>" + sStart + sReturn.replaceAll("\\\\n", "<br>") + sEnd + "</body></html>";
        } else {
        	sReturn = sStart + sReturn + sEnd;
        }
		return sReturn;
	}
}
