package allrounder.plugin;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.TreeMap;

import allrounder.DbHandler;
import allrounder.DirHandler;


public class PluginLoader {
	private static PluginLoader ref;
	private final static String PluginDirectory = "plugins";
	private TreeMap<String, YiffIndexPlugin> tmLoadedPlugins = new TreeMap<String, YiffIndexPlugin>();
	
	private PluginLoader() {
		try {
			addDirectoryToClasspath(DirHandler.getAppDataDir());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		this.tmLoadedPlugins = loadPluginsFromDB();
	}
	
	public synchronized void destruct() {
		this.tmLoadedPlugins = null;
	}
	
	public static synchronized PluginLoader getInstance() {
		if (ref == null) {
			ref = new PluginLoader();
		}
		return ref;
	}
	
	//just to be sure
	@Override
	public Object clone() throws CloneNotSupportedException {
	    throw new CloneNotSupportedException(); 
	}
	
	public static TreeMap<String, YiffIndexPlugin> getAllAviablePlugins() {
		TreeMap<String, YiffIndexPlugin> tmAviablePlugins = new TreeMap<String, YiffIndexPlugin>();
		tmAviablePlugins.putAll(getAllAviablePluginsOfDir(new File(DirHandler.getAppDir(), PluginDirectory)));
		
		File fTmp = new File(DirHandler.getAppDataDir(), PluginDirectory);
		if (fTmp.exists() && fTmp.isDirectory()) tmAviablePlugins.putAll(getAllAviablePluginsOfDir(fTmp));
		return tmAviablePlugins;
	}
	
	private static TreeMap<String, YiffIndexPlugin> getAllAviablePluginsOfDir(File dir) {
		TreeMap<String, YiffIndexPlugin> tmAviablePlugins = new TreeMap<String, YiffIndexPlugin>();
		
		// get all class files
		ArrayList<File> alClassFiles = findClassFilesInDir(dir);
		
		// load all that are loadable
		for (File fClassFile : alClassFiles) {
			String sLoaderPath = fClassFile.getAbsolutePath();
			
			// remove .class
			sLoaderPath = sLoaderPath.substring(0, sLoaderPath.length() - 6);
			
			// remove everything before /plugins/
			int iTmp = sLoaderPath.indexOf(File.separator + PluginDirectory + File.separator);
			sLoaderPath = sLoaderPath.substring(iTmp + 1);
			
			// replace / with .
			while (sLoaderPath.contains(File.separator)) {
				iTmp = sLoaderPath.indexOf(File.separator);
				sLoaderPath = sLoaderPath.substring(0, iTmp) + "." + sLoaderPath.substring(iTmp + 1);
			}
			
			YiffIndexPlugin yPlugin = null;
			// try to load the class as a YiffIndexPlugin
			try {
				yPlugin = (YiffIndexPlugin)ClassLoader.getSystemClassLoader().loadClass(sLoaderPath).newInstance();
			} catch (Exception e) {
				yPlugin = null; // not a plugin
				//e.printStackTrace();
				continue;
			}
			
			tmAviablePlugins.put(sLoaderPath, yPlugin);
		}
		return tmAviablePlugins;
	}
	
	public static TreeMap<String, YiffIndexPlugin> loadPluginsFromDB() {
		TreeMap<String, YiffIndexPlugin> tmPlugins = new TreeMap<String, YiffIndexPlugin>();
		ArrayList<String> pluginNames = DbHandler.getInstance().getPugins();
		
		for (String s : pluginNames) {
			YiffIndexPlugin yPlugin = null;
			try {
				yPlugin = (YiffIndexPlugin)ClassLoader.getSystemClassLoader().loadClass(s).newInstance();
			} catch (Exception e) {
				// plugin was deleted, remove from db
				DbHandler.getInstance().removePlugin(s);
				continue;
			}
			tmPlugins.put(s, yPlugin);
		}
		
		return tmPlugins;
	}
	
	public void reloadPluginsFromDB() {
		this.tmLoadedPlugins = loadPluginsFromDB();
	}
	
	public TreeMap<String, YiffIndexPlugin> getLoadedPlugins() {
		return this.tmLoadedPlugins;
	}
	
	private static ArrayList<File> findClassFilesInDir(File directory) {
		ArrayList<File> alClassFiles = new ArrayList<File>();
		for (File f : directory.listFiles()) {
			if (f.isDirectory()) {
				alClassFiles.addAll(findClassFilesInDir(f));
				continue;
			}
			if (f.isFile() && f.getName().length() > 6 && f.getName().substring(f.getName().length() - 6).equalsIgnoreCase(".class")) {
				alClassFiles.add(f);
			}
		}
		return alClassFiles;
	}
	
	@SuppressWarnings("unchecked")
	private static final Class[] parameters = new Class[]{URL.class};
	 
	private static void addDirectoryToClasspath(File dir) throws IOException {
		if (dir == null || !dir.exists() || !dir.isDirectory()) return;
		addURLToClasspath(dir.toURI().toURL());
	}//end method
	 
	 
	private static void addURLToClasspath(URL u) throws IOException {
		URLClassLoader sysloader = (URLClassLoader)ClassLoader.getSystemClassLoader();
		Class<URLClassLoader> sysclass = URLClassLoader.class;
		
		for (URL url : sysloader.getURLs()) {
			if (url.equals(u)) return; // allready in classpath
		}
	 
		try {
			Method method = sysclass.getDeclaredMethod("addURL",parameters);
			method.setAccessible(true);
			method.invoke(sysloader,new Object[]{ u });
		} catch (Throwable t) {
			t.printStackTrace();
			throw new IOException("Error, could not add URL to system classloader");
		}//end try catch
			
	}//end method
}
