package backup.local;

import index.IndexPanel;

import java.awt.Cursor;
import java.io.File;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import allrounder.DbHandler;
import allrounder.StringLib;
import allrounder.TaggedImage;

public class Export {
	public static void doExport(ArrayList<TaggedImage> images, IndexPanel pnl) {
		pnl.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		// display filechooser and make sure that a) an allready existing file is not accidentally overwritten
		// and b) the file ends with .yiff (muha)
		JFileChooser chooser = new JFileChooser();
		chooser.setFileFilter(new FileFilter() {
			@Override
			public boolean accept(File arg0) {
				if (arg0.getName().endsWith("." + StringLib.getInstance().getValue("backup_local.filefilter.extension")) ||
						arg0.isDirectory()) {
					return true;
				}
				return false;
			}
			@Override
			public String getDescription() {
				return StringLib.getInstance().getValue("backup_local.filefilter.descr");
			}
		});
		int returnVal = chooser.showSaveDialog(null);
		
	    if(returnVal != JFileChooser.APPROVE_OPTION) return;
	    
	    File exFileName = chooser.getSelectedFile();
	    if (!exFileName.getName().endsWith("." + StringLib.getInstance().getValue("backup_local.filefilter.extension"))) {
	    	exFileName = new File(exFileName.getAbsolutePath() + "." + StringLib.getInstance().getValue("backup_local.filefilter.extension"));
	    }
	    if (exFileName.exists()) {
	    	int i = JOptionPane.showConfirmDialog(null, StringLib.getInstance().getValue("backup_local.Export.fileexistquestion.text"), StringLib.getInstance().getValue("backup_local.Export.fileexistquestion.title"), JOptionPane.YES_NO_OPTION);
	    	if (i != JOptionPane.YES_OPTION) return;
	    }
		
	    // if the given parameter is null, export /ALL/ images from DB
		if (images == null) {
			images = new ArrayList<TaggedImage>();
			ArrayList<Integer> ids = DbHandler.getInstance().getAllImageIds();
			for (int i : ids) {
				images.add(DbHandler.getInstance().getImageFromImageId(i));
			}
		}
		
		// change the array list to an map to make sure that each hash is only exportet once.
		// maybe too much just for downsizing the export. if there are speed issues, i'll have to remove that
		TreeMap<String, TaggedImage> hashMap = new TreeMap<String, TaggedImage>();
		for (TaggedImage img : images) {
			hashMap.put(img.hash, img);
		}
		
		try {
			ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(exFileName));
			oos.writeDouble(Double.parseDouble(DbHandler.getInstance().getGeneral("Version"))); // version of YiffIndex
			oos.writeInt(hashMap.size()); // save the size for importing
			Iterator<String> iter = hashMap.keySet().iterator();
			while (iter.hasNext()) {
				// save the images without the arraylist (or map) for two puropses:
				// 1. no arraylist overhead
				// 2. possibility to display an acurate progressbar while importing
				oos.writeObject(hashMap.get(iter.next()));
			}
			oos.close();
			
			JOptionPane.showMessageDialog(null, StringLib.getInstance().getValue("backup_local.Export.finished.text", "", " " + hashMap.size()), StringLib.getInstance().getValue("backup_local.Export.finished.title"), JOptionPane.INFORMATION_MESSAGE);
		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, StringLib.getInstance().getValue("backup_local.Export.serializeerror.text") + e.getLocalizedMessage(), StringLib.getInstance().getValue("backup_local.Export.serializeerror.title"), JOptionPane.ERROR_MESSAGE);
		} finally {
			pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}
}
