package index;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.ImageTypeSpecifier;
import javax.imageio.ImageWriter;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.metadata.IIOMetadataNode;
import javax.imageio.stream.ImageOutputStream;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;

import org.w3c.dom.NodeList;

import allrounder.StringLib;
import allrounder.logger.Logwriter;

import com.sun.imageio.plugins.gif.GIFImageMetadata;
import com.sun.jimi.core.Jimi;
import com.sun.jimi.core.JimiException;

public class ImagePanel extends JPanel implements ActionListener {
	private static final long serialVersionUID = -2383392159667289465L;
	ImageToolPanel imgTools;
	ImageScrollPane scrollPane;
	boolean bFit = true;
	ArrayList<ActionListener> listener = new ArrayList<ActionListener>();
	File actualImage = null;
	
	public ImagePanel() {
		this.imgTools = new ImageToolPanel(this);
		setEnabledBack(false);
		setEnabledNext(false);
		double[][] layout = {{TableLayoutConstants.FILL},{TableLayoutConstants.FILL, 30}};
		setLayout(new TableLayout(layout));
		this.scrollPane = new ImageScrollPane();
		this.scrollPane.getViewport().setBackground(new Color(51, 51, 51));
		
		this.scrollPane.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				// recall open image. everything else is done there
				if (ImagePanel.this.actualImage != null) openImage(ImagePanel.this.actualImage);
			}
		});
		
		add(this.scrollPane, "0, 0");
		add(this.imgTools, "0, 1");
	}
	
	public void openImage(File img) {
		this.actualImage = img;
		this.scrollPane.removeImage();
		
		if (this.actualImage == null) return;
		Image awtImg = readImgFromFile(img, this.scrollPane.getViewport());
		
		// check if image is smaller than actual viewport size
		// if so, deactivate the zoom buttons and display whole image
		Dimension d = this.scrollPane.getViewport().getSize();
		if (awtImg.getWidth(this.scrollPane.getViewport()) < d.getWidth() && awtImg.getHeight(this.scrollPane.getViewport()) < d.getHeight()) {
			this.imgTools.btnFit.setEnabled(false);
			this.imgTools.btnMaximize.setEnabled(false);
		} else if (this.bFit) {
			this.imgTools.btnMaximize.setEnabled(true);
			awtImg = scaleToFit(awtImg, this.scrollPane.getViewport().getSize(), Image.SCALE_FAST, this.scrollPane.getViewport());
		} else {
			this.imgTools.btnFit.setEnabled(true);
		}
		this.scrollPane.setImage(awtImg);
	}
	
	public static Image readImgFromFile(File img, Component drawTarget) {
		if (img == null) return null;
		Image awtImg = null;
		
		try {
			// special case for animated gifs
			// funnily, loading them into an awt image with Jimi seems impossible (only into an
			// enumeration of the single images)
			if (img.getName().substring(img.getName().length() - 4).equalsIgnoreCase(".gif")) {
				// fix for bug when delaytime is 0 (set it to 10 instead)
				ImageReader irImgRead = ImageIO.getImageReadersByFormatName("gif").next();
				irImgRead.setInput(ImageIO.createImageInputStream(img));
				
				// just check first frame
				GIFImageMetadata metadatazero = (GIFImageMetadata)irImgRead.getImageMetadata(0);
				if (metadatazero.delayTime != 0) {
					awtImg = Toolkit.getDefaultToolkit().createImage(img.getAbsolutePath());
				} else {
					// write new image to byte[]
					ImageWriter iwImgWrit = ImageIO.getImageWriter(irImgRead);
					ByteArrayOutputStream baoStream = new ByteArrayOutputStream();
					ImageOutputStream ios = ImageIO.createImageOutputStream(baoStream);
					iwImgWrit.setOutput(ios);
					iwImgWrit.prepareWriteSequence(null);
					for (int i = 0; i < irImgRead.getNumImages(true); ++i) {
						BufferedImage src = irImgRead.read(i);     

						//prepare new metadata
						IIOMetadata metadata = iwImgWrit.getDefaultImageMetadata(new ImageTypeSpecifier(src), null);
						
						// get old metadata
						IIOMetadataNode root = (IIOMetadataNode)irImgRead.getImageMetadata(i).getAsTree("javax_imageio_gif_image_1.0");
						
						// add delay
						NodeList children = root.getChildNodes();
						for (int j = 0; j < children.getLength(); ++j) {
							if (!children.item(j).getNodeName().equals("GraphicControlExtension")) continue;
							IIOMetadataNode gce = (IIOMetadataNode)children.item(j);
							gce.setAttribute("delayTime", "10");
							break;
						}
						
						// set new metadata
						metadata.setFromTree(metadata.getNativeMetadataFormatName(), root);

						IIOImage ii = new IIOImage(src, null, metadata);
						iwImgWrit.writeToSequence( ii, iwImgWrit.getDefaultWriteParam());
					}
					iwImgWrit.endWriteSequence();
					ios.close();
					
					awtImg = Toolkit.getDefaultToolkit().createImage(baoStream.toByteArray());
				}
			} else {
				awtImg = Jimi.getImage(img.getAbsolutePath());
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		MediaTracker mdr = new MediaTracker(drawTarget);
		mdr.addImage(awtImg, 0);
		try {
			mdr.waitForID(0);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		return awtImg;
	}
	
	public static Image scaleToFit(Image img, Dimension d, int mode, Component drawTarget) {
		int imgWidth = img.getWidth(drawTarget);
		int imgHeight = img.getHeight(drawTarget);
		
		if (imgWidth <= d.width || d.height <= imgHeight/((double)imgWidth/(double)d.width)) {
			imgHeight = imgHeight < d.height ? imgHeight : d.height;
			imgWidth = -1;
		} else {
			imgWidth = imgWidth < d.width ? imgWidth : d.width;
			imgHeight = - 1;
		}
		
		Image returnImg = img.getScaledInstance(imgWidth, imgHeight, mode);
		MediaTracker mdr = new MediaTracker(drawTarget);
		mdr.addImage(returnImg, 0);
		try {
			mdr.waitForID(0);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		return returnImg;
	}
	
	public void removeImage() {
		this.actualImage = null;
		this.scrollPane.removeImage(); //.getViewport().removeAll();
		setEnabledBack(false);
		setEnabledNext(false);
	}
	
	public void setEnabledBack(boolean b) {
		this.imgTools.setEnabledBack(b);
	}
	
	public void setEnabledNext(boolean b) {
		this.imgTools.setEnabledNext(b);
	}
	
	public void addImageChangeActionListener(ActionListener a) {
		this.listener.add(a);
	}
	
	public int getImgWidth() {
		if (this.actualImage == null || !this.actualImage.exists()) return 0;
		try {
			return Jimi.getRasterImage(this.actualImage.getAbsolutePath()).getWidth();
		} catch (JimiException e) {
			e.printStackTrace();
		}
		return 0;
	}
	
	public int getImgHeight() {
		if (this.actualImage == null || !this.actualImage.exists()) return 0;
		try {
			return Jimi.getRasterImage(this.actualImage.getAbsolutePath()).getHeight();
		} catch (JimiException e) {
			e.printStackTrace();
		}
		return 0;
	}
	
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if(command.equalsIgnoreCase("MAXIMIZE")) {
			this.imgTools.swapSizeButtons();
			this.bFit = false;
			openImage(this.actualImage);
		} else if (command.equalsIgnoreCase("FIT")) {
			this.imgTools.swapSizeButtons();
			this.bFit = true;
			openImage(this.actualImage);
		} else if (command.equalsIgnoreCase("BACK") || command.equalsIgnoreCase("NEXT")) {
			for (ActionListener al : this.listener) al.actionPerformed(e);
		}
		
	}
}

class ImageToolPanel extends JPanel {
	private static final long serialVersionUID = 2094083876622303906L;
	JButton btnMaximize;
	JButton btnFit;
	JButton btnBack;
	JButton btnNext;
	ActionListener actList;
	StringLib lib = StringLib.getInstance();
	public ImageToolPanel(ActionListener al) {
		this.actList = al;
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		add(Box.createHorizontalGlue());
		this.btnMaximize = makeButton("general/ZoomIn24", "MAXIMIZE", this.lib.getValue("index.ImagePanel.maximize.tt"), this.lib.getValue("index.ImagePanel.maximize.alt"));
		add(this.btnMaximize);
		this.btnFit = makeButton("general/ZoomOut24", "FIT", this.lib.getValue("index.ImagePanel.fit.tt"), this.lib.getValue("index.ImagePanel.fit.alt"));
		this.btnFit.setEnabled(false);
		add(this.btnFit);
		add(Box.createHorizontalGlue());
		this.btnBack = makeButton("navigation/Back24", "BACK", this.lib.getValue("index.ImagePanel.back.tt"), this.lib.getValue("index.ImagePanel.back.alt"));
		this.btnBack.setMnemonic(KeyEvent.VK_LEFT);
		add(this.btnBack);
		this.btnNext = makeButton("navigation/Forward24", "NEXT", this.lib.getValue("index.ImagePanel.next.tt"), this.lib.getValue("index.ImagePanel.next.alt"));
		this.btnNext.setMnemonic(KeyEvent.VK_RIGHT);
		add(this.btnNext);
		add(Box.createHorizontalGlue());
	}
	
	private JButton makeButton(String imageName,
			String actionCommand, String toolTipText, String altText) {
		// Look for the image.
		String imgLocation = "/toolbarButtonGraphics/" + imageName + ".gif";
		URL imageURL = getClass().getResource(imgLocation);

		// Create and initialize the button.
		JButton button = new JButton();
		button.setActionCommand(actionCommand);
		button.setToolTipText(toolTipText);
		button.addActionListener(this.actList);

		if (imageURL != null) { // image found
			button.setIcon(new ImageIcon(imageURL, altText));
		} else { // no image found
			button.setText(altText);
			Logwriter.getInstance().writeLine("Resource not found: " + imgLocation, true);
		}

		return button;
	}
	
	public void setEnabledBack(boolean b) {
		this.btnBack.setEnabled(b);
	}
	
	public void setEnabledNext(boolean b) {
		this.btnNext.setEnabled(b);
	}
	
	public void swapSizeButtons() {
		this.btnMaximize.setEnabled(!this.btnMaximize.isEnabled());
		this.btnFit.setEnabled(!this.btnFit.isEnabled());
	}
}
