package index;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.event.ContainerAdapter;
import java.awt.event.ContainerEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.ScrollPaneConstants;
import javax.swing.ScrollPaneLayout;
import javax.swing.Scrollable;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;
import javax.swing.event.MouseInputAdapter;
import javax.swing.event.MouseInputListener;

public class ImageScrollPane extends JScrollPane {
	private static final long serialVersionUID = 7625608502795112250L;
	ImageIcon icon = null;

	public ImageScrollPane() {
		ScrollPaneSelector.installScrollPaneSelector(this);
		this.getVerticalScrollBar().setUnitIncrement(20);
	}
	
	public void setImage(Image img) {
		if (this.icon != null && this.icon.getImage() != null) this.icon.getImage().flush();
		this.icon = new ImageIcon(img);
		GrabAndScrollLabel gasl = new GrabAndScrollLabel(this.icon);
		this.setViewportView(gasl);
		System.gc();
	}
	
	public void removeImage() {
		this.getViewport().removeAll();
		if (this.icon != null && this.icon.getImage() != null) this.icon.getImage().flush();
		System.gc();
	}
}


// GRAB-AND-DRAG-SCROLLING
// from: http://www.javafaq.nu/java-bookpage-18-2.html
class GrabAndScrollLabel extends JLabel
{
	private static final long serialVersionUID = -2333718764451799083L;

	public GrabAndScrollLabel(ImageIcon i) {
		super(i);
		MouseInputAdapter mia = new MouseInputAdapter() {
			int m_XDifference, m_YDifference;
			Container c;
			@Override
			public void mouseDragged(MouseEvent e) {
				this.c = GrabAndScrollLabel.this.getParent();
				if (this.c instanceof JViewport) {
					JViewport jv = (JViewport) this.c;
					Point p = jv.getViewPosition();
					int newX = p.x - (e.getX() - this.m_XDifference);
					int newY = p.y - (e.getY() - this.m_YDifference);
					int maxX = GrabAndScrollLabel.this.getWidth()
					- jv.getWidth();
					int maxY = GrabAndScrollLabel.this.getHeight()
					- jv.getHeight();
					if (newX < 0)
						newX = 0;
					if (newX > maxX)
						newX = maxX;
					if (newY < 0)
						newY = 0;
					if (newY > maxY)
						newY = maxY;
					jv.setViewPosition(new Point(newX, newY));
				}
			}
			
			@Override
			public void mousePressed(MouseEvent e) {
				setCursor(Cursor.getPredefinedCursor(
				Cursor.MOVE_CURSOR));
				this.m_XDifference = e.getX();
				this.m_YDifference = e.getY();
			}
			
			@Override
			public void mouseReleased(MouseEvent e) {
				setCursor(Cursor.getPredefinedCursor(
				Cursor.DEFAULT_CURSOR));
			}
		};

		addMouseMotionListener(mia);
		addMouseListener(mia);
	}
}

//IMAGE-PREVIEW
//from: https://jdnc-incubator.dev.java.net/source/browse/jdnc-incubator/src/weebib/java/org/jdesktop/swingx/scrollpaneselector/
/*
 * $Id: ScrollPaneSelector.java,v 1.1 2006/07/29 01:56:05 weebib Exp $
 *
 * Copyright 2004 Sun Microsystems, Inc., 4150 Network Circle,
 * Santa Clara, California 95054, U.S.A. All rights reserved.
 */

/**
 * ScrollPaneSelector is a little utility class that provides a means to quickly scroll both vertically and
 * horizontally on a single mouse click, by dragging a selection rectangle over a "thumbnail" of the
 * scrollPane's viewport view.
 * <p>
 * Once the selector is installed on a given JScrollPane instance, a little button appears as soon as at least
 * one of its scroll bars is made visible.
 * @author weebib
 */
class ScrollPaneSelector extends JComponent {
	private static final long serialVersionUID = -4606529810241932409L;
	// static final fields
	private static final double MAX_SIZE = 200;
	private static final Icon LAUNCH_SELECTOR_ICON = new Icon() {
		public void paintIcon(Component c, Graphics g, int x, int y) {
			Color tmpColor = g.getColor();
			g.setColor(Color.BLACK);
			g.drawRect(2, 2, 10, 10);
			g.drawRect(4, 5, 6, 4);
			g.setColor(tmpColor);
		}

		public int getIconWidth() {
			return 15;
		}

		public int getIconHeight() {
			return 15;
		}
	};
	private static Map<JScrollPane, ScrollPaneSelector> theInstalledScrollPaneSelectors
			= new HashMap<JScrollPane, ScrollPaneSelector>();
	private static final String COMPONENT_ORIENTATION = "componentOrientation";

	// member fields
	private LayoutManager theFormerLayoutManager;
	JScrollPane theScrollPane;
	JComponent theComponent;
	JPopupMenu thePopupMenu;
	JComponent theButton;
	private BufferedImage theImage;
	Rectangle theStartRectangle;
	Rectangle theRectangle;
	Point theStartPoint;
	double theScale;
	private PropertyChangeListener theComponentOrientationListener;
	private ContainerAdapter theViewPortViewListener;

	/**
	 * Installs a ScrollPaneSelector to the given JScrollPane instance.
	 * @param aScrollPane
	 */
	public synchronized static void installScrollPaneSelector(JScrollPane aScrollPane) {
		if (aScrollPane == null) return;
		if (theInstalledScrollPaneSelectors.containsKey(aScrollPane)) return;
		ScrollPaneSelector scrollPaneSelector = new ScrollPaneSelector();
		scrollPaneSelector.installOnScrollPane(aScrollPane);
		theInstalledScrollPaneSelectors.put(aScrollPane, scrollPaneSelector);
	}

	/**
	 * Removes the ScrollPaneSelector from the given JScrollPane.
	 * @param aScrollPane
	 */
	public synchronized static void uninstallScrollPaneSelector(JScrollPane aScrollPane) {
		if (aScrollPane == null) return;
		ScrollPaneSelector scrollPaneSelector = theInstalledScrollPaneSelectors.get(aScrollPane);
		if (scrollPaneSelector == null) return;
		scrollPaneSelector.uninstallFromScrollPane();
		theInstalledScrollPaneSelectors.remove(aScrollPane);
	}

//-- Constructor ------
	private ScrollPaneSelector() {
		setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
		this.theScrollPane = null;
		this.theImage = null;
		this.theStartRectangle = null;
		this.theRectangle = null;
		this.theStartPoint = null;
		this.theScale = 0.0;
		this.theButton = new JLabel(LAUNCH_SELECTOR_ICON);
		MouseInputListener mil = new MouseInputAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				Point p = e.getPoint();
				SwingUtilities.convertPointToScreen(p, ScrollPaneSelector.this.theButton);
				display(p);
			}

			@Override
			public void mouseReleased(MouseEvent e) {
				if (ScrollPaneSelector.this.theStartPoint != null) {
					Point newPoint = e.getPoint();
					SwingUtilities.convertPointToScreen(newPoint, (Component)e.getSource());
					int deltaX = (int)((newPoint.x - ScrollPaneSelector.this.theStartPoint.x) / ScrollPaneSelector.this.theScale);
					int deltaY = (int)((newPoint.y - ScrollPaneSelector.this.theStartPoint.y) / ScrollPaneSelector.this.theScale);
					scroll(deltaX, deltaY);
				}
				ScrollPaneSelector.this.theStartPoint = null;
				ScrollPaneSelector.this.theStartRectangle = ScrollPaneSelector.this.theRectangle;
			}

			@Override
			public void mouseDragged(MouseEvent e) {
				if (ScrollPaneSelector.this.theStartPoint == null) return;
				Point newPoint = e.getPoint();
				SwingUtilities.convertPointToScreen(newPoint, (Component)e.getSource());
				moveRectangle(newPoint.x - ScrollPaneSelector.this.theStartPoint.x, newPoint.y - ScrollPaneSelector.this.theStartPoint.y);
			}
		};
		this.theButton.addMouseListener(mil);
		this.theButton.addMouseMotionListener(mil);
		setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
		this.thePopupMenu = new JPopupMenu();
		this.thePopupMenu.setLayout(new BorderLayout());
		this.thePopupMenu.add(this, BorderLayout.CENTER);
		this.theComponentOrientationListener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if (ScrollPaneSelector.this.theScrollPane == null) return;
				ScrollPaneSelector.this.theScrollPane.setCorner(ScrollPaneConstants.LOWER_LEADING_CORNER, null);
				ScrollPaneSelector.this.theScrollPane.setCorner(ScrollPaneConstants.LOWER_TRAILING_CORNER, ScrollPaneSelector.this.theButton);
			}
		};
		this.theViewPortViewListener = new ContainerAdapter() {
			@Override
			public void componentAdded(ContainerEvent e) {
				if (ScrollPaneSelector.this.thePopupMenu.isVisible()) ScrollPaneSelector.this.thePopupMenu.setVisible(false);
				Component comp = ScrollPaneSelector.this.theScrollPane.getViewport().getView();
				ScrollPaneSelector.this.theComponent = (comp instanceof JComponent)? (JComponent)comp : null;
			}
		};
	}

//-- JComponent overriden methods ------
	@Override
	public Dimension getPreferredSize() {
		if (this.theImage == null || this.theRectangle == null) return new Dimension();
		Insets insets = getInsets();
		return new Dimension(this.theImage.getWidth(null) + insets.left + insets.right,
							 this.theImage.getHeight(null) + insets.top + insets.bottom);
	}

	@Override
	protected void paintComponent(Graphics g1D) {
		if (this.theImage == null || this.theRectangle == null) return;
		Graphics2D g = (Graphics2D)g1D;
		Insets insets = getInsets();
		int xOffset = insets.left;
		int yOffset = insets.top;
		int availableWidth = getWidth() - insets.left - insets.right;
		int availableHeight = getHeight() - insets.top - insets.bottom;
		g.drawImage(this.theImage, xOffset, yOffset, null);
		Color tmpColor = g.getColor();
		Area area = new Area(new Rectangle(xOffset, yOffset, availableWidth, availableHeight));
		area.subtract(new Area(this.theRectangle));
		g.setColor(new Color(200, 200, 200, 128));
		g.fill(area);
		g.setColor(Color.BLACK);
		g.draw(this.theRectangle);
		g.setColor(tmpColor);
	}

//-- Private methods ------
	private void installOnScrollPane(JScrollPane aScrollPane) {
		if (this.theScrollPane != null) uninstallFromScrollPane();
		this.theScrollPane = aScrollPane;
		this.theFormerLayoutManager = this.theScrollPane.getLayout();
		this.theScrollPane.setLayout(new TweakedScrollPaneLayout());
		this.theScrollPane.addPropertyChangeListener(COMPONENT_ORIENTATION, this.theComponentOrientationListener);
		this.theScrollPane.getViewport().addContainerListener(this.theViewPortViewListener);
		this.theScrollPane.setCorner(ScrollPaneConstants.LOWER_TRAILING_CORNER, this.theButton);
		Component comp = this.theScrollPane.getViewport().getView();
		this.theComponent = (comp instanceof JComponent)? (JComponent)comp : null;
	}

	private void uninstallFromScrollPane() {
		if (this.theScrollPane == null) return;
		if (this.thePopupMenu.isVisible()) this.thePopupMenu.setVisible(false);
		this.theScrollPane.setCorner(ScrollPaneConstants.LOWER_TRAILING_CORNER, null);
		this.theScrollPane.removePropertyChangeListener(COMPONENT_ORIENTATION, this.theComponentOrientationListener);
		this.theScrollPane.getViewport().removeContainerListener(this.theViewPortViewListener);
		this.theScrollPane.setLayout(this.theFormerLayoutManager);
		this.theScrollPane = null;
	}

	void display(Point aPointOnScreen) {
		if (this.theComponent == null) return;
		double compWidth = this.theComponent.getWidth();
		double compHeight = this.theComponent.getHeight();
		double scaleX = MAX_SIZE / compWidth;
		double scaleY = MAX_SIZE / compHeight;
		this.theScale = Math.min(scaleX, scaleY);
		this.theImage = new BufferedImage((int)(this.theComponent.getWidth() * this.theScale),
									 (int)(this.theComponent.getHeight() * this.theScale),
									 BufferedImage.TYPE_INT_RGB);

		Graphics2D g = this.theImage.createGraphics();
		g.scale(this.theScale, this.theScale);
		this.theComponent.paint(g);
		this.theStartRectangle = this.theComponent.getVisibleRect();
		Insets insets = getInsets();
		this.theStartRectangle.x = (int)(this.theScale * this.theStartRectangle.x + insets.left);
		this.theStartRectangle.y = (int)(this.theScale * this.theStartRectangle.y + insets.right);
		this.theStartRectangle.width *= this.theScale;
		this.theStartRectangle.height *= this.theScale;
		this.theRectangle = this.theStartRectangle;

		Dimension pref = this.thePopupMenu.getPreferredSize();
		Point buttonLocation = this.theButton.getLocationOnScreen();
		Point popupLocation = new Point((this.theButton.getWidth() - pref.width) / 2,
										(this.theButton.getHeight() - pref.height) / 2);
		Point centerPoint = new Point(buttonLocation.x + popupLocation.x + this.theRectangle.x + this.theRectangle.width / 2,
									  buttonLocation.y + popupLocation.y + this.theRectangle.y + this.theRectangle.height / 2);
		try {
			// Attempt to move the mouse pointer to the center of the selector's rectangle.
			new Robot().mouseMove(centerPoint.x, centerPoint.y);
			this.theStartPoint = centerPoint;
		} catch (Exception e) {
			// Since we cannot move the cursor, we'll move the popup instead.
			this.theStartPoint = aPointOnScreen;
			popupLocation.x += this.theStartPoint.x - centerPoint.x;
			popupLocation.y += this.theStartPoint.y - centerPoint.y;
		}
		this.thePopupMenu.show(this.theButton, popupLocation.x, popupLocation.y);
	}

	void moveRectangle(int aDeltaX, int aDeltaY) {
		if (this.theStartRectangle == null) return;

		Insets insets = getInsets();
		Rectangle newRect = new Rectangle(this.theStartRectangle);
		newRect.x += aDeltaX;
		newRect.y += aDeltaY;
		newRect.x = Math.min(Math.max(newRect.x, insets.left), getWidth() - insets.right - newRect.width);
		newRect.y = Math.min(Math.max(newRect.y, insets.right), getHeight() - insets.bottom - newRect.height);
		Rectangle clip = new Rectangle();
		Rectangle2D.union(this.theRectangle, newRect, clip);
		clip.grow(2, 2);
		this.theRectangle = newRect;
		paintImmediately(clip);
	}

	void scroll(int aDeltaX, int aDeltaY) {
		if (this.theComponent == null) return;
		Rectangle rect = this.theComponent.getVisibleRect();
		rect.x += aDeltaX;
		rect.y += aDeltaY;
		this.theComponent.scrollRectToVisible(rect);
		this.thePopupMenu.setVisible(false);
	}
}

/*
 * $Id: TweakedScrollPaneLayout.java,v 1.1 2006/07/29 01:56:06 weebib Exp $
 *
 * Copyright 2004 Sun Microsystems, Inc., 4150 Network Circle,
 * Santa Clara, California 95054, U.S.A. All rights reserved.
 */
/**
 * A hideous hack to allow the display of the selector's button even when only one scrollBar is visible.
 *
 * @author weebib
 */
class TweakedScrollPaneLayout extends ScrollPaneLayout {
	private static final long serialVersionUID = -5569509010510083829L;

	private void superlayoutContainer(Container parent) {
		JScrollPane scrollPane = (JScrollPane)parent;
		this.vsbPolicy = scrollPane.getVerticalScrollBarPolicy();
		this.hsbPolicy = scrollPane.getHorizontalScrollBarPolicy();

		Rectangle availR = scrollPane.getBounds();
		Insets insets = parent.getInsets();
		availR.x = insets.left;
		availR.y = insets.top;
		availR.width -= insets.left + insets.right;
		availR.height -= insets.top + insets.bottom;

		boolean leftToRight = scrollPane.getComponentOrientation().isLeftToRight();

		Rectangle colHeadR = new Rectangle(0, availR.y, 0, 0);

		if ((this.colHead != null) && (this.colHead.isVisible())) {
			int colHeadHeight = Math.min(availR.height, this.colHead.getPreferredSize().height);
			colHeadR.height = colHeadHeight;
			availR.y += colHeadHeight;
			availR.height -= colHeadHeight;
		}

		Rectangle rowHeadR = new Rectangle(0, 0, 0, 0);

		if ((this.rowHead != null) && (this.rowHead.isVisible())) {
			int rowHeadWidth = Math.min(availR.width, this.rowHead.getPreferredSize().width);
			rowHeadR.width = rowHeadWidth;
			availR.width -= rowHeadWidth;
			if (leftToRight) {
				rowHeadR.x = availR.x;
				availR.x += rowHeadWidth;
			} else {
				rowHeadR.x = availR.x + availR.width;
			}
		}

		Border viewportBorder = scrollPane.getViewportBorder();
		Insets vpbInsets;
		if (viewportBorder != null) {
			vpbInsets = viewportBorder.getBorderInsets(parent);
			availR.x += vpbInsets.left;
			availR.y += vpbInsets.top;
			availR.width -= vpbInsets.left + vpbInsets.right;
			availR.height -= vpbInsets.top + vpbInsets.bottom;
		} else {
			vpbInsets = new Insets(0, 0, 0, 0);
		}

		Component view = (this.viewport != null) ? this.viewport.getView() : null;
		Dimension viewPrefSize = (view != null) ? view.getPreferredSize() : new Dimension(0, 0);

		Dimension extentSize = (this.viewport != null) ? this.viewport.toViewCoordinates(availR.getSize()) : new Dimension(0, 0);

		boolean viewTracksViewportWidth = false;
		boolean viewTracksViewportHeight = false;
		boolean isEmpty = (availR.width < 0 || availR.height < 0);
		Scrollable sv;
		if (!isEmpty && view instanceof Scrollable) {
			sv = (Scrollable)view;
			viewTracksViewportWidth = sv.getScrollableTracksViewportWidth();
			viewTracksViewportHeight = sv.getScrollableTracksViewportHeight();
		} else {
			sv = null;
		}

		Rectangle vsbR = new Rectangle(0, availR.y - vpbInsets.top, 0, 0);

		boolean vsbNeeded;
		if (isEmpty) {
			vsbNeeded = false;
		} else if (this.vsbPolicy == VERTICAL_SCROLLBAR_ALWAYS) {
			vsbNeeded = true;
		} else if (this.vsbPolicy == VERTICAL_SCROLLBAR_NEVER) {
			vsbNeeded = false;
		} else {  // vsbPolicy == VERTICAL_SCROLLBAR_AS_NEEDED
			vsbNeeded = !viewTracksViewportHeight && (viewPrefSize.height > extentSize.height);
		}

		if ((this.vsb != null) && vsbNeeded) {
			adjustForVSB(true, availR, vsbR, vpbInsets, leftToRight);
			extentSize = this.viewport.toViewCoordinates(availR.getSize());
		}

		Rectangle hsbR = new Rectangle(availR.x - vpbInsets.left, 0, 0, 0);
		boolean hsbNeeded;
		if (isEmpty) {
			hsbNeeded = false;
		} else if (this.hsbPolicy == HORIZONTAL_SCROLLBAR_ALWAYS) {
			hsbNeeded = true;
		} else if (this.hsbPolicy == HORIZONTAL_SCROLLBAR_NEVER) {
			hsbNeeded = false;
		} else {  // hsbPolicy == HORIZONTAL_SCROLLBAR_AS_NEEDED
			hsbNeeded = !viewTracksViewportWidth && (viewPrefSize.width > extentSize.width);
		}
		if ((this.hsb != null) && hsbNeeded) {
			adjustForHSB(true, availR, hsbR, vpbInsets);
			if ((this.vsb != null) && !vsbNeeded && (this.vsbPolicy != VERTICAL_SCROLLBAR_NEVER)) {
				extentSize = this.viewport.toViewCoordinates(availR.getSize());
				vsbNeeded = viewPrefSize.height > extentSize.height;
				if (vsbNeeded) {
					adjustForVSB(true, availR, vsbR, vpbInsets, leftToRight);

					// Hack to correct Bug 6411225
					// I don't actually measure the consequences of this bug "fix"

					if (!leftToRight) {
						hsbR.x += vsbR.width;
					}

					// end of hack
				}
			}
		}

		if (this.viewport != null) {
			this.viewport.setBounds(availR);

			if (sv != null) {
				extentSize = this.viewport.toViewCoordinates(availR.getSize());

				boolean oldHSBNeeded = hsbNeeded;
				boolean oldVSBNeeded = vsbNeeded;
				viewTracksViewportWidth = sv.
						getScrollableTracksViewportWidth();
				viewTracksViewportHeight = sv.
						getScrollableTracksViewportHeight();
				if (this.vsb != null && this.vsbPolicy == VERTICAL_SCROLLBAR_AS_NEEDED) {
					boolean newVSBNeeded = !viewTracksViewportHeight && (viewPrefSize.height > extentSize.height);
					if (newVSBNeeded != vsbNeeded) {
						vsbNeeded = newVSBNeeded;
						adjustForVSB(vsbNeeded, availR, vsbR, vpbInsets, leftToRight);
						extentSize = this.viewport.toViewCoordinates(availR.getSize());
					}
				}
				if (this.hsb != null && this.hsbPolicy == HORIZONTAL_SCROLLBAR_AS_NEEDED) {
					boolean newHSBbNeeded = !viewTracksViewportWidth && (viewPrefSize.width > extentSize.width);
					if (newHSBbNeeded != hsbNeeded) {
						hsbNeeded = newHSBbNeeded;
						adjustForHSB(hsbNeeded, availR, hsbR, vpbInsets);
						if ((this.vsb != null) && !vsbNeeded && (this.vsbPolicy != VERTICAL_SCROLLBAR_NEVER)) {
							extentSize = this.viewport.toViewCoordinates(availR.getSize());
							vsbNeeded = viewPrefSize.height > extentSize.height;
							if (vsbNeeded) {
								adjustForVSB(true, availR, vsbR, vpbInsets, leftToRight);
							}
						}
					}
				}
				if (oldHSBNeeded != hsbNeeded || oldVSBNeeded != vsbNeeded) {
					this.viewport.setBounds(availR);
				}
			}
		}
		vsbR.height = availR.height + vpbInsets.top + vpbInsets.bottom;
		hsbR.width = availR.width + vpbInsets.left + vpbInsets.right;
		rowHeadR.height = availR.height + vpbInsets.top + vpbInsets.bottom;
		rowHeadR.y = availR.y - vpbInsets.top;
		colHeadR.width = availR.width + vpbInsets.left + vpbInsets.right;
		colHeadR.x = availR.x - vpbInsets.left;

		if (this.rowHead != null) {
			this.rowHead.setBounds(rowHeadR);
		}

		if (this.colHead != null) {
			this.colHead.setBounds(colHeadR);
		}

		if (this.vsb != null) {
			if (vsbNeeded) {
				this.vsb.setVisible(true);
				this.vsb.setBounds(vsbR);
			} else {
				this.vsb.setVisible(false);
			}
		}

		if (this.hsb != null) {
			if (hsbNeeded) {
				this.hsb.setVisible(true);
				this.hsb.setBounds(hsbR);
			} else {
				this.hsb.setVisible(false);
			}
		}

		//bug 6411225 ????????

		if (this.lowerLeft != null) {
			this.lowerLeft.setBounds(leftToRight ? rowHeadR.x : vsbR.x,
								hsbR.y,
								leftToRight ? rowHeadR.width : vsbR.width,
								hsbR.height);
		}

		if (this.lowerRight != null) {
			this.lowerRight.setBounds(leftToRight ? vsbR.x : rowHeadR.x,
								 hsbR.y,
								 leftToRight ? vsbR.width : rowHeadR.width,
								 hsbR.height);
		}

		if (this.upperLeft != null) {
			this.upperLeft.setBounds(leftToRight ? rowHeadR.x : vsbR.x,
								colHeadR.y,
								leftToRight ? rowHeadR.width : vsbR.width,
								colHeadR.height);
		}

		if (this.upperRight != null) {
			this.upperRight.setBounds(leftToRight ? vsbR.x : rowHeadR.x,
								 colHeadR.y,
								 leftToRight ? vsbR.width : rowHeadR.width,
								 colHeadR.height);
		}
	}

	private void adjustForVSB(boolean wantsVSB, Rectangle available, Rectangle vsbR,
							  Insets vpbInsets, boolean leftToRight) {
		int oldWidth = vsbR.width;
		if (wantsVSB) {
			int vsbWidth = Math.max(0, Math.min(this.vsb.getPreferredSize().width,
												available.width));

			available.width -= vsbWidth;
			vsbR.width = vsbWidth;

			if (leftToRight) {
				vsbR.x = available.x + available.width + vpbInsets.right;
			} else {
				vsbR.x = available.x - vpbInsets.left;
				available.x += vsbWidth;
			}
		} else {
			available.width += oldWidth;
		}
	}

	private void adjustForHSB(boolean wantsHSB, Rectangle available,
							  Rectangle hsbR, Insets vpbInsets) {
		int oldHeight = hsbR.height;
		if (wantsHSB) {
			int hsbHeight = Math.max(0, Math.min(available.height,
												 this.hsb.getPreferredSize().height));

			available.height -= hsbHeight;
			hsbR.y = available.y + available.height + vpbInsets.bottom;
			hsbR.height = hsbHeight;
		} else {
			available.height += oldHeight;
		}
	}

	@Override
	public void layoutContainer(Container parent) {
		superlayoutContainer(parent);
		boolean isLeftToRight = parent.getComponentOrientation().isLeftToRight();
		if (isLeftToRight && this.lowerRight == null) return;
		if (!isLeftToRight && this.lowerLeft == null) return;
		if (this.vsb.isVisible() && !this.hsb.isVisible()) {
			Rectangle vsbBounds = this.vsb.getBounds();
			Dimension hsbPref = this.hsb.getPreferredSize();
			int delta = hsbPref.height;
			vsbBounds.height -= delta;
			this.vsb.setBounds(vsbBounds);
			if (isLeftToRight) {
				this.lowerRight.setBounds(vsbBounds.x, vsbBounds.y + vsbBounds.height, vsbBounds.width, delta);
				this.lowerRight.setVisible(true);
			} else {
				this.lowerLeft.setBounds(vsbBounds.x, vsbBounds.y + vsbBounds.height, vsbBounds.width, delta);
				this.lowerLeft.setVisible(true);
			}
		}
		if (this.hsb.isVisible() && !this.vsb.isVisible()) {
			Rectangle hsbBounds = this.hsb.getBounds();
			Dimension vsbPref = this.vsb.getPreferredSize();
			int delta = vsbPref.width;
			hsbBounds.width -= delta;
			if (!isLeftToRight) {
				hsbBounds.x += delta;
			}
			this.hsb.setBounds(hsbBounds);
			if (isLeftToRight) {
				this.lowerRight.setBounds(hsbBounds.x + hsbBounds.width, hsbBounds.y, delta, hsbBounds.height);
				this.lowerRight.setVisible(true);
			} else {
				this.lowerLeft.setBounds(hsbBounds.x - delta, hsbBounds.y, delta, hsbBounds.height);
				this.lowerLeft.setVisible(true);
			}
		}
	}
}