package index;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import allrounder.DbHandler;
import allrounder.TaggedImage;
import allrounder.md5.Hasher;

public class IndexPanel extends JPanel implements ActionListener, ListSelectionListener {
	private static final long serialVersionUID = 1462274540600540892L;
	FilePanel filePanel = new FilePanel();
	ImagePanel imgPanel = new ImagePanel();
	TagPanel tagPanel = new TagPanel();
	boolean bUpdate = false;
	public IndexPanel() {
		setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        					//column / row
		double[][] layout = {{0.3, 10, TableLayoutConstants.FILL},{TableLayoutConstants.FILL, 25, this.tagPanel.getPreferredSize().height}};
		this.setLayout(new TableLayout(layout));
		this.add(this.filePanel, "0, 0");
		this.add(this.imgPanel, "2, 0, 2, 0");
		this.add(this.tagPanel, "0, 2, 2, 2");
		//this.add(btnSave, "3, 4");

		this.filePanel.addListSelectionListener(this);
		this.imgPanel.addImageChangeActionListener(this);
	}
	
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equalsIgnoreCase("BACK")) {
			this.filePanel.previous();
		} else if (arg0.getActionCommand().equalsIgnoreCase("NEXT")) {
			this.filePanel.next();
		}
	}
	
	public FilePanel getFilePanel() {
		return this.filePanel;
	}
	
	public TagPanel getTagPanel() {
		return this.tagPanel;
	}
	
	public ImagePanel getImagePanel() {
		return this.imgPanel;
	}
	
	/*private void save() {
		save(filePanel.getImage());
	}*/
	
	public void save(File f) {
		if (!f.exists()) return;
		boolean bWasChangedBefore = DbHandler.getInstance().getImageFromPath(f).changed;
		TaggedImage img = this.tagPanel.getTags();
		img.path = f;
		img.hash = Hasher.imageToHash(f);
		img.width = this.imgPanel.getImgWidth();
		img.height = this.imgPanel.getImgHeight();
		img.changed = (this.tagPanel.hasChanged()) ? true : bWasChangedBefore;
		save(img);
	}
	
	private void save(TaggedImage img) {
		System.out.println("save");
		this.bUpdate = true;
		if (img.isEmpty()) {
			DbHandler.getInstance().removeImageFromDatabase(img.path);
			this.filePanel.setFileStateNotIndexed(img.path);
		} else {
			DbHandler.getInstance().addImageToDatabase(img);
			this.filePanel.setFileStateIndexed(img.path);
		}
	}

	public void valueChanged(ListSelectionEvent arg0) {
		if(arg0.getValueIsAdjusting()) return;
		if (this.filePanel.getImage() == null) {
			this.imgPanel.removeImage();
			this.tagPanel.setAllEnabled(false);
			return;
		}
		
		// save if the value has changed
		if (this.tagPanel.hasChanged() && arg0.getFirstIndex() != -1) {
			// skip if changed from nowhere or reselected the same
			if (arg0.getFirstIndex() != -1 && arg0.getFirstIndex() != arg0.getLastIndex()) {
				// yes, i know, the whole "save when selection changed" here is an ugly hack, but i diddn't know a better way.
				// if you know one, feel free to contact me
				//get the prev file
				int lower = arg0.getFirstIndex();
				int higher = arg0.getLastIndex();
				save(this.filePanel.getImage().equals(this.filePanel.getImageAt(lower)) ? this.filePanel.getImageAt(higher) : this.filePanel.getImageAt(lower));
			}
		}
		
		if (this.bUpdate) {
			this.tagPanel.reset();
			this.bUpdate = false;
			this.filePanel.updateList();
			
			// updatelist sets a selection in the list, therefore this method will be called
			// again so we can return here
			return;
		}
		this.imgPanel.openImage(this.filePanel.getImage());
		this.imgPanel.setEnabledBack(this.filePanel.hasPrevious());
		this.imgPanel.setEnabledNext(this.filePanel.hasNext());
		TaggedImage img = DbHandler.getInstance().getImageFromPath(this.filePanel.getImage());
		this.tagPanel.setTags(img);
		
		// NOTE: To be removed if loading an untagged image takes too long with a large database
		if (img.isEmpty() && this.filePanel.getImage().exists()) {
			// image adress not found in DB. check for image hash (in case a duplicate of the image is allready in the DB)
			img = DbHandler.getInstance().getImageFromHash(Hasher.imageToHash(this.filePanel.getImage()));
			if (img != null) {
				this.tagPanel.setTags(img);
				save(this.filePanel.getImage());
			}
		}
	}
}
