package info;

import index.IndexPanel;
import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Color;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.URI;
import java.util.ArrayList;
import java.util.Locale;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import allrounder.DbHandler;
import allrounder.DirHandler;
import allrounder.StringLib;

public class InfoDialog extends JDialog {
	private static final long serialVersionUID = 4494535472785020097L;
	StringLib lib = StringLib.getInstance();
	JLabel lblYiffIndex = new JLabel(this.lib.getValue("info.YiffIndex"));
	JLabel lblVersion = new JLabel(this.lib.getValue("info.Version") + " " + DbHandler.getInstance().getGeneral("Version"));
	JLabel lblCreator = new JLabel(this.lib.getValue("info.Creator"));
	JLabel lblLangDE = new JLabel(this.lib.getValue("info.lang.de"));
	JLabel lblLangEN = new JLabel(this.lib.getValue("info.lang.en"));
	JLabel lblSupport = new JLabel(this.lib.getValue("info.support"));
	JLabel lblLogo = new JLabel(this.lib.getValue("info.logo"));
	JLabel lblBeta1 = new JLabel(this.lib.getValue("info.Betatester.v1"));
	JLabel lblBeta2 = new JLabel(this.lib.getValue("info.Betatester.v2"));
	JLabel lblServer = new JLabel(this.lib.getValue("info.Server"));
	JButton btnHelp = new JButton(this.lib.getValue("info.help"));
	JButton btnHomepage = new JButton(this.lib.getValue("info.web"));
	
	public InfoDialog(IndexPanel indexPanel) {		
		this.btnHelp.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				// open homepage in the system's default browser
				// java is to dumb to do this alone, but i can take BrowserLauncher2
				File fHelpFile = new File(DirHandler.getAppPath() + File.separator + "help" + File.separator + Locale.getDefault().getLanguage() + ".html");
				
				if (!fHelpFile.exists()) {
					// default to english language
					fHelpFile = new File(DirHandler.getAppPath() + File.separator + "help" + File.separator + "en.html");
					
					if (!fHelpFile.exists()) {
						//abort
						JOptionPane.showMessageDialog(InfoDialog.this, "Fatal error: help not found", "Seriously, WTF is going on?", JOptionPane.ERROR_MESSAGE);
						return;
					}
				}
				try {
					if (Desktop.isDesktopSupported()) {
						Desktop desktop = Desktop.getDesktop();
						desktop.browse(fHelpFile.toURI());
					} else {
						throw new Exception();
					}
				} catch (Exception e) {
					JOptionPane.showMessageDialog(InfoDialog.this, InfoDialog.this.lib.getValue("info.browserlaunchererror.text") + "\n" + fHelpFile.getAbsolutePath(), InfoDialog.this.lib.getValue("info.browserlaunchererror.title"), JOptionPane.ERROR_MESSAGE);
				}
			}
		});
		
		this.btnHomepage.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				// open homepage in the system's default browser
				// java is to dumb to do this alone, but i can take BrowserLauncher2
				try {
					if (Desktop.isDesktopSupported()) {
						Desktop desktop = Desktop.getDesktop();
						desktop.browse(new URI("http://yiffindex.net.tf/"));
					} else {
						throw new Exception();
					}
				} catch (Exception e) {
					JOptionPane.showMessageDialog(InfoDialog.this, InfoDialog.this.lib.getValue("info.browserlaunchererror.text") + "\nhttp://yiffindex.net.tf/", InfoDialog.this.lib.getValue("info.browserlaunchererror.title"), JOptionPane.ERROR_MESSAGE);
				}
			}
		});
		
		setTitle(this.lib.getValue("info.title"));
		((JPanel)this.getContentPane()).setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		this.lblYiffIndex.setFont(new Font("Serif", Font.BOLD, 36));
		this.lblYiffIndex.setForeground(Color.RED);
		this.lblYiffIndex.setPreferredSize(new Dimension(this.lblYiffIndex.getPreferredSize().width + 1, this.lblYiffIndex.getPreferredSize().height));
		this.lblYiffIndex.setMinimumSize(this.lblYiffIndex.getPreferredSize());

		int iLabelHeight = this.lblVersion.getPreferredSize().height;
		int iButtonHeight = this.btnHelp.getPreferredSize().height;
		
		setSize(500, 350);
		setLocationRelativeTo(indexPanel);
		//setResizable(false);
		
		//column / row
		double[][] layout = {{TableLayoutConstants.FILL},{this.lblYiffIndex.getPreferredSize().height, 5, iLabelHeight, 5, iLabelHeight, 5, iLabelHeight, 5, iLabelHeight, 5, iLabelHeight, 5, iLabelHeight, 5, iLabelHeight, 5, iLabelHeight, 5, iLabelHeight, TableLayoutConstants.FILL, iButtonHeight, 5, iButtonHeight}};
		this.setLayout(new TableLayout(layout));

		this.add(this.lblYiffIndex, "0, 0, c, c");
		this.add(this.lblVersion, "0, 2, c, c");
		this.add(this.lblCreator, "0, 4");
		this.add(this.lblLangDE, "0, 6");
		this.add(this.lblLangEN, "0, 8");
		this.add(this.lblSupport, "0, 10");
		this.add(this.lblLogo, "0, 12");
		this.add(this.lblBeta1, "0, 14");
		this.add(this.lblBeta2, "0, 16");
		this.add(this.lblServer, "0, 18");
		this.add(this.btnHelp, "0, 20");
		this.add(this.btnHomepage, "0, 22");
		
		setVisible(true);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	}
	
	@SuppressWarnings("unused")
	private ArrayList<File> getAllJavaFiles(File directory) {
		ArrayList<File> JavaFiles = new ArrayList<File>();
		for (File f : directory.listFiles()) {
			if (f.isDirectory()) {
				JavaFiles.addAll(getAllJavaFiles(f));
			} else {
				if (f.getAbsolutePath().endsWith(".java")) JavaFiles.add(f);
			}
		}
		
		return JavaFiles;
	}
}
