package search;

import index.FilePanel;
import index.ImagePanel;
import index.ImageScrollPane;
import index.TagPanel;
import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.JButton;
import javax.swing.JDialog;

import allrounder.DbHandler;
import allrounder.StringLib;
import allrounder.TaggedImage;

public class ImageUpdater extends JDialog implements ActionListener {
	private static final long serialVersionUID = -89317142764756344L;
	JButton btnSave = new JButton(StringLib.getInstance().getValue("search.ImageUpdate.btnSave.text"));
	JButton btnClose = new JButton(StringLib.getInstance().getValue("search.ImageUpdate.btnClose.text"));
	ImageScrollPane imgPane = new ImageScrollPane();
	TagPanel tagPanel = new TagPanel();
	TaggedImage img = new TaggedImage();
	FilePanel filePanel;
	
	public ImageUpdater(TaggedImage img, FilePanel fp) {
		this.filePanel = fp;
		this.img = img;
		this.btnSave.setActionCommand("SAVE");
		this.btnSave.addActionListener(this);
		this.btnClose.setActionCommand("CLOSE");
		this.btnClose.addActionListener(this);
		
		// row/column
		double[][] layout = {{this.btnSave.getPreferredSize().width, 10, this.btnClose.getPreferredSize().width, TableLayoutConstants.FILL},{TableLayoutConstants.FILL, this.tagPanel.getPreferredSize().height, 10, this.btnSave.getPreferredSize().height}};
		setLayout(new TableLayout(layout));
		
		add(this.imgPane, "0, 0, 3, 0");
		add(this.tagPanel, "0, 1, 3, 0");
		add(this.btnSave, "0, 3");
		add(this.btnClose, "2, 3");
		
		Image awtImg = ImagePanel.readImgFromFile(img.path, this.imgPane);

		setSize(850, 600);
		setVisible(true);
		awtImg = ImagePanel.scaleToFit(awtImg, this.imgPane.getViewport().getSize(), Image.SCALE_DEFAULT, this.imgPane);
		this.imgPane.setImage(awtImg);
		this.tagPanel.setTags(img);
	
		addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				Image awtImg = ImagePanel.readImgFromFile(ImageUpdater.this.img.path, ImageUpdater.this.imgPane);
				awtImg = ImagePanel.scaleToFit(awtImg, ImageUpdater.this.imgPane.getViewport().getSize(), Image.SCALE_DEFAULT, ImageUpdater.this.imgPane);
				ImageUpdater.this.imgPane.setImage(awtImg);
			}
		});
		
		//setModalityType(ModalityType.APPLICATION_MODAL);
		setTitle(StringLib.getInstance().getValue("search.ImageUpdate.title"));
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setVisible(true);
	}

	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equalsIgnoreCase("SAVE")) {
			TaggedImage saveImg = new TaggedImage();
			
			saveImg = this.tagPanel.getTags();
			saveImg.hash = this.img.hash;
			saveImg.width = this.img.width;
			saveImg.height = this.img.height;
			saveImg.path = this.img.path;
			saveImg.changed = true;
			if (saveImg.isEmpty()) {
				DbHandler.getInstance().removeImageFromDatabase(saveImg.path);
				this.filePanel.setFileStateNotIndexed(saveImg.path);
			} else {
				DbHandler.getInstance().addImageToDatabase(saveImg);
				this.filePanel.setFileStateIndexed(saveImg.path);
			}
			
			setVisible(false);
			dispose();
		} else if (arg0.getActionCommand().equalsIgnoreCase("CLOSE")) {
			setVisible(false);
			dispose();
		}
	}
}
