package search;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import allrounder.StringLib;
import allrounder.TaggedImage;

public class ResolutionSearchField extends JPanel implements SearchInterface, ActionListener, FocusListener {
	private static final long serialVersionUID = -4865292095984816623L;
	boolean bAddCloseButton = true;
	JButton btnClose = CloseIcon.getCloseButton();
	JLabel lblWidth = new JLabel(StringLib.getInstance().getValue("search.ResolutionSearchField.width"));
	JComboBox jcbWidth;
	JLabel lblHeight = new JLabel(StringLib.getInstance().getValue("search.ResolutionSearchField.height"));
	JComboBox jcbHeight;
	JLabel lblOr = new JLabel(StringLib.getInstance().getValue("search.SearchInterface.or"));
	JTextField jtfWidth = new JTextField("0");
	JTextField jtfHeight = new JTextField("0");
	Vector<ActionListener> listeners = new Vector<ActionListener>();
	
	public ResolutionSearchField(boolean b, ActionListener al) {
		this.bAddCloseButton = b;
		this.listeners.add(al);
		construct();
	}
	
	public ResolutionSearchField(ActionListener al) {
		this.listeners.add(al);
		construct();
	}
	
	public void addActionListener(ActionListener al) {
		this.listeners.add(al);
	}
	
	private void construct() {
		String[] range = {"<=", "=", ">="};
		this.jcbWidth = new JComboBox(range);
		this.jtfWidth.addFocusListener(this);
		this.jcbHeight = new JComboBox(range);
		this.jtfHeight.addFocusListener(this);
		
		//column / row
		double[][] layout = {{this.btnClose.getPreferredSize().width + 10, 10, this.lblOr.getPreferredSize().width, 10, this.lblWidth.getPreferredSize().width, 10, this.jcbWidth.getPreferredSize().width, 5, TableLayoutConstants.FILL, 20, this.lblHeight.getPreferredSize().width, 10, this.jcbHeight.getPreferredSize().width, 5, TableLayoutConstants.FILL},{TableLayoutConstants.FILL, this.jcbWidth.getPreferredSize().height, TableLayoutConstants.FILL}};
		this.setLayout(new TableLayout(layout));
		
		if (this.bAddCloseButton) {
			this.btnClose.setActionCommand("SUBCLOSE");
			this.btnClose.addActionListener(this);
			this.btnClose.setToolTipText(StringLib.getInstance().getValue("search.SearchCriteriaPanel.btnClose.tt"));
			add(this.btnClose, "0, 1, c, c");
			add(this.lblOr, "2, 1, c, c");
		}
		
		add(this.lblWidth, "4, 1");
		add(this.jcbWidth, "6, 1");
		add(this.jtfWidth, "8, 1");
		add(this.lblHeight, "10, 1");
		add(this.jcbHeight, "12, 1");
		add(this.jtfHeight, "14, 1");
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				ResolutionSearchField.this.jtfWidth.requestFocusInWindow();
				ResolutionSearchField.this.jtfWidth.selectAll();
			}
		});
	}
	
	@Override
	public TreeSet<TaggedImage> filterImages(TreeSet<TaggedImage> images) {
		TreeSet<TaggedImage> resultSet = new TreeSet<TaggedImage>();
		Iterator<TaggedImage> it = images.iterator();
		while (it.hasNext()) {
			TaggedImage img = it.next();
			if (isValid(img)) {
				resultSet.add(img);
			}
		}
		
		return resultSet;
	}
	
	private boolean isValid(TaggedImage img) {
		boolean bWidthOK = false;
		String s = (String)this.jcbWidth.getSelectedItem();
		int width = 0;
		int height = 0;
		try {
			width = Integer.parseInt(this.jtfWidth.getText());
		} catch(NumberFormatException e) {
			this.jtfWidth.setText("0");
		}
		try {
			height = Integer.parseInt(this.jtfHeight.getText());
		} catch(NumberFormatException e) {
			this.jtfHeight.setText("0");
		}
		
		if (s.equals("<=")) {
			if (img.width <= width) bWidthOK = true;
		} else if (s.equals("=")) {
			if (img.width == width) bWidthOK = true;
		} else if (s.equals(">=")) {
			if (img.width >= width) bWidthOK = true;
		}
		
		boolean bHeightOK = false;
		s = (String)this.jcbHeight.getSelectedItem();
		if (s.equals("<=")) {
			if (img.height <= height) bHeightOK = true;
		} else if (s.equals("=")) {
			if (img.height == height) bHeightOK = true;
		} else if (s.equals(">=")) {
			if (img.height >= height) bHeightOK = true;
		}
		
		return (bWidthOK && bHeightOK);
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equalsIgnoreCase("SUBCLOSE")) {
			arg0.setSource(this);
			for (ActionListener al : this.listeners) {
				al.actionPerformed(arg0);
			}
		}
	}

	@Override
	public void focusGained(FocusEvent arg0) {}

	@Override
	public void focusLost(FocusEvent arg0) {
		if(arg0.getSource() == this.jtfWidth) {
			try {
				Integer.parseInt(this.jtfWidth.getText());
			} catch(NumberFormatException e) {
				this.jtfWidth.setText("0");
			}
		} else {
			try {
				Integer.parseInt(this.jtfHeight.getText());
			} catch(NumberFormatException e) {
				this.jtfHeight.setText("0");
			}
		}
	}

}
