package search;

import index.FilePanel;
import index.ImagePanel;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.Collections;
import java.util.TreeSet;

import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;
import allrounder.DbHandler;
import allrounder.StringLib;
import allrounder.TaggedImage;

public class ResultScrollPane extends JScrollPane {
	private static final long serialVersionUID = -3313998632665700393L;
	FilePanel filePanel;
	JTable tblResults;
	StringLib lib = StringLib.getInstance();
	
	public ResultScrollPane(FilePanel fp) {
		this.filePanel = fp;
	}
	
	public void setImages(TreeSet<TaggedImage> images) {
		if (images == null || images.size() == 0) {
			JLabel lblNoResults = new JLabel(this.lib.getValue("search.ResultScrollPane.noresults"));
			setViewportView(lblNoResults);
			this.tblResults = null;
			return;
		}
		String[] columnNames = {this.lib.getValue("search.ResultScrollPane.column.name"),
				this.lib.getValue("search.ResultScrollPane.column.artist"),
				this.lib.getValue("search.ResultScrollPane.column.headcount"),
				this.lib.getValue("search.ResultScrollPane.column.spezies"),
				this.lib.getValue("search.ResultScrollPane.column.gender"),
				this.lib.getValue("search.ResultScrollPane.column.yiff"),
				this.lib.getValue("search.ResultScrollPane.column.fetish"),
				this.lib.getValue("search.ResultScrollPane.column.other"),
				this.lib.getValue("search.ResultScrollPane.column.resolution")};
		
		Object[][] data = new Object[images.size()][columnNames.length];
		
		int i = 0;
		for (TaggedImage img : images) {
			data[i][0] = new FileWrapper(img.path);
			
			StringBuilder artists = new StringBuilder("");
			for (String s : img.artists) {
				artists.append(s + ", ");
			}
			if (artists.length() != 0) artists.delete(artists.length() - 2, artists.length());
			data[i][1] = artists.toString();
			
			data[i][2] = img.headcount;
			
			StringBuilder spezies = new StringBuilder("");
			for (int id : img.spezies) {
				spezies.append(this.lib.getValue("species." + id) + ", ");
			}
			if (spezies.length() != 0) spezies.delete(spezies.length() - 2, spezies.length());
			data[i][3] = spezies.toString();
			
			StringBuilder gender = new StringBuilder("");
			for (int id : img.gender) {
				gender.append(this.lib.getValue("Tag.sex." + id + ".text") + ", ");
			}
			if (gender.length() != 0) gender.delete(gender.length() - 2, gender.length());
			data[i][4] = gender.toString();
			
			data[i][5] = img.yiff ? this.lib.getValue("search.ResultScrollPane.column.yiff.yes") : this.lib.getValue("search.ResultScrollPane.column.yiff.no");
			
			StringBuilder fetish = new StringBuilder("");
			for (int id : img.fetish) {
				fetish.append(this.lib.getValue("fetishchooser.fetishes." + id + ".text") + ", ");
			}
			if (fetish.length() != 0) fetish.delete(fetish.length() - 2, fetish.length());
			data[i][6] = fetish.toString();
			
			StringBuilder other = new StringBuilder("");
			for (String s : img.other) {
				other.append(s + ", ");
			}
			if (other.length() != 0) other.delete(other.length() - 2, other.length());
			data[i][7] = other.toString();
			data[i][8] = new ResolutionWrapper(img.width, img.height);
			++i;
		}
		
		TableModel mdl = new DefaultTableModel(data, columnNames) {
			private static final long serialVersionUID = 7562031132130959931L;

			@Override
			public boolean isCellEditable(int row, int column) {
				return false;
			}
			
			@Override
			public Class<?> getColumnClass(int c) {
		        return getValueAt(0, c).getClass();
		    }
		};
		
		this.tblResults = new JTable(mdl){
			private static final long serialVersionUID = 4020626251489963700L;

			@Override
			public TableCellRenderer getCellRenderer(int row, int column) {
		        return new MyRenderer();
		    }
		};
		
		MyMouseMotionListener mmml = new MyMouseMotionListener(this.tblResults, this.filePanel);
		this.tblResults.addMouseMotionListener(mmml);
		this.tblResults.addMouseListener(mmml);
		this.tblResults.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		this.tblResults.setAutoCreateRowSorter(true);
		//TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(); 
		//tblResults.setRowSorter( sorter );
        //sorter.setModel( tblResults.getModel() );
		setViewportView(this.tblResults);
	}
}


class MyRenderer extends DefaultTableCellRenderer /*implements TableCellRenderer*/ {
	private static final long serialVersionUID = 5210062776483877425L;

	public MyRenderer() {
		setOpaque(true); // MUST do this for background to show up.
	}
	 
	@Override
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		if (value instanceof FileWrapper) {
			JLabel lbl = (JLabel)super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column); //new JLabel(((FileWrapper)value).getName());
			lbl.setToolTipText(((FileWrapper)value).getFile().getAbsolutePath());
			return lbl;
		}
		return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
	}
}

class MyMouseMotionListener implements MouseMotionListener, MouseListener, ActionListener {
	JTable tblSource;
	JFrame imageFrame = new JFrame("thumb_frm");
	JLabel lblImage = new JLabel(new ImageIcon());
	int actualRow = -1;
	int imgSize = 250;

	FilePanel filePanel;
	JPopupMenu popupMenuMulti = new JPopupMenu();
	JPopupMenu popupMenuSingle = new JPopupMenu();
	Point pMenuSource = new Point();
	StringLib lib = StringLib.getInstance();
	
	MyMouseMotionListener(JTable source, FilePanel fp) {
		this.imageFrame.add(this.lblImage);
		this.imageFrame.setSize(this.imgSize, this.imgSize);
		this.imageFrame.setUndecorated(true);
		this.imageFrame.setAlwaysOnTop(true);
		this.tblSource = source;
		this.filePanel = fp;
		
		// contextmenu
		JMenuItem jmi = buildMenuItem(this.lib.getValue("search.ResultScrollPane.slideAct"), "SLIDEACT");
		jmi.setFont(jmi.getFont().deriveFont(jmi.getFont().getStyle() | Font.BOLD));
		JMenuItem jmi2 = buildMenuItem(this.lib.getValue("search.ResultScrollPane.slideAct"), "SLIDEACT");
		jmi2.setFont(jmi2.getFont().deriveFont(jmi2.getFont().getStyle() | Font.BOLD));
		this.popupMenuMulti.add(jmi);
		this.popupMenuSingle.add(jmi2);
		this.popupMenuMulti.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.copyUrl"), "COPYURL"));
		this.popupMenuSingle.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.copyUrl"), "COPYURL"));
		this.popupMenuMulti.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.copyInfo"), "COPYINFO"));
		this.popupMenuSingle.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.copyInfo"), "COPYINFO"));
		this.popupMenuMulti.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.copyActImg"), "COPYACTIMG"));
		this.popupMenuSingle.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.copyActImg"), "COPYACTIMG"));
		this.popupMenuMulti.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.editActImg"), "EDITACTIMG"));
		this.popupMenuSingle.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.editActImg"), "EDITACTIMG"));
		this.popupMenuMulti.add(new JPopupMenu.Separator());
		this.popupMenuMulti.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.slideSel"), "SLIDESEL"));
		this.popupMenuMulti.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.slideSelRandom"), "SLIDESELRAND"));
		this.popupMenuMulti.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.copySelImgs"), "COPYSELIMGS"));
		this.popupMenuMulti.add(new JPopupMenu.Separator());
		this.popupMenuSingle.add(new JPopupMenu.Separator());
		this.popupMenuMulti.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.slideAll"), "SLIDEALL"));
		this.popupMenuSingle.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.slideAll"), "SLIDEALL"));
		this.popupMenuMulti.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.slideAllRandom"), "SLIDEALLRAND"));
		this.popupMenuSingle.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.slideAllRandom"), "SLIDEALLRAND"));
		this.popupMenuMulti.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.copyAllImgs"), "COPYALLIMGS"));
		this.popupMenuSingle.add(buildMenuItem(this.lib.getValue("search.ResultScrollPane.copyAllImgs"), "COPYALLIMGS"));
	}
	
	private JMenuItem buildMenuItem(String text, String actionCommand) {
		JMenuItem menuItem = new JMenuItem(text);
		menuItem.setActionCommand(actionCommand);
		menuItem.addActionListener(this);
		menuItem.setFont(menuItem.getFont().deriveFont(menuItem.getFont().getStyle() ^ Font.BOLD));
		return menuItem;
	}
	
	public void mouseDragged(MouseEvent arg0) {}

	public void mouseMoved(MouseEvent arg0) {
		Point p = arg0.getPoint();
		
		int row = this.tblSource.rowAtPoint(p);
		if (row == this.actualRow) return;
		this.actualRow = row;

		File imgPath = ((FileWrapper)this.tblSource.getValueAt(row, this.tblSource.convertColumnIndexToView(0))).getFile();

		Image awtImg = ImagePanel.readImgFromFile(imgPath, this.lblImage);
		
		Dimension d = new Dimension(this.imgSize, this.imgSize);
		int imgWidth = awtImg.getWidth(this.lblImage);
		int imgHeight = awtImg.getHeight(this.lblImage);
		
		if (imgWidth <= d.width || d.height <= imgHeight/((double)imgWidth/(double)d.width)) {
			imgHeight = imgHeight < d.height ? imgHeight : d.height;
			imgWidth = -1;
		} else {
			imgWidth = imgWidth < d.width ? imgWidth : d.width;
			imgHeight = - 1;
		}
		awtImg = awtImg.getScaledInstance(imgWidth, imgHeight, Image.SCALE_FAST);
		
		this.lblImage.setIcon(new ImageIcon(awtImg));
		//imageFrame.add(imageLabel);
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		Point tblPoint = this.tblSource.getParent().getLocationOnScreen(); // location of the JViewPort
		int dif = this.tblSource.getTableHeader().getHeight() + this.imgSize;
		if (tblPoint.y >= dif) {
			this.imageFrame.setLocation(new Point(tblPoint.x, tblPoint.y - dif));
		} else {
			this.imageFrame.setLocation(new Point(tblPoint.x, 0));
		}
		this.imageFrame.setVisible(true);
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		this.imageFrame.setVisible(false);
		this.actualRow = -1;
	}

	public void mouseClicked(MouseEvent arg0) {
		 if (arg0.getClickCount() == 2){
	        int row = this.tblSource.rowAtPoint(arg0.getPoint());
	        ArrayList<File> al = new ArrayList<File>();
	        al.add(((FileWrapper)this.tblSource.getValueAt(row, this.tblSource.convertColumnIndexToView(0))).getFile());
	        new SlideShow(al);
	     }
	}
	
	public void mousePressed(MouseEvent arg0) {
		showPopup(arg0);
	}
	public void mouseReleased(MouseEvent arg0) {
		showPopup(arg0);
	}
	
	private void showPopup(MouseEvent e) {
		if (e.isPopupTrigger()) {
			this.pMenuSource = e.getPoint();
			if (getSelFiles().size() <= 1) {
				this.popupMenuSingle.show(e.getComponent(), e.getX(), e.getY());
			} else {
				this.popupMenuMulti.show(e.getComponent(), e.getX(), e.getY());
			}
		}
	}

	public void actionPerformed(ActionEvent arg0) {
		TaggedImage img = DbHandler.getInstance().getImageFromPath(getActFile().get(0));
		if (img == null || img.isEmpty()) {
			// TODO: translate
			JOptionPane.showMessageDialog(null, "Image not availlable", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		if (arg0.getActionCommand().equalsIgnoreCase("EDITACTIMG")) {
			new ImageUpdater(DbHandler.getInstance().getImageFromPath(getActFile().get(0)), this.filePanel);
		} else if (arg0.getActionCommand().equalsIgnoreCase("SLIDEACT")) {
			new SlideShow(getActFile());
		} else if (arg0.getActionCommand().equalsIgnoreCase("SLIDESEL")) {
			ArrayList<File> al = getSelFiles();
			if (al.size() == 0) return;
			new SlideShow(al);
		} else if (arg0.getActionCommand().equalsIgnoreCase("SLIDESELRAND")) {
			ArrayList<File> al = getSelFiles();
			if (al.size() == 0) return;
			Collections.shuffle(al);
			new SlideShow(al);
		} else if (arg0.getActionCommand().equalsIgnoreCase("SLIDEALL")) {
			ArrayList<File> al = getAllFiles();
			if (al.size() == 0) return;
			new SlideShow(al);
		} else if (arg0.getActionCommand().equalsIgnoreCase("SLIDEALLRAND")) {
			ArrayList<File> al = getAllFiles();
			if (al.size() == 0) return;
			Collections.shuffle(al);
			new SlideShow(al);
		} else if (arg0.getActionCommand().equalsIgnoreCase("COPYACTIMG")) {
			copyImages(getActFile());
		} else if (arg0.getActionCommand().equalsIgnoreCase("COPYSELIMGS")) {
			ArrayList<File> al = getSelFiles();
			if (al.size() == 0) return;
			copyImages(al);
		} else if (arg0.getActionCommand().equalsIgnoreCase("COPYALLIMGS")) {
			copyImages(getAllFiles());
		} else if (arg0.getActionCommand().equalsIgnoreCase("COPYURL")) {
			copyUrl(getActFile());
		} else if (arg0.getActionCommand().equalsIgnoreCase("COPYINFO")) {
			copyInfo();
		}
	}

	private ArrayList<File> getActFile() {
		ArrayList<File> al = new ArrayList<File>();
		al.add(((FileWrapper)this.tblSource.getValueAt(this.tblSource.rowAtPoint(this.pMenuSource), this.tblSource.convertColumnIndexToView(0))).getFile());
		return al;
	}
	
	private ArrayList<File> getSelFiles() {
		int selRowCount = this.tblSource.getSelectedRowCount();
		if (selRowCount == 0) return new ArrayList<File>();

		ArrayList<File> al = new ArrayList<File>();
		if (selRowCount == 1) {
			al.add(((FileWrapper)this.tblSource.getValueAt(this.tblSource.getSelectedRow(), this.tblSource.convertColumnIndexToView(0))).getFile());
			return al;
		}
		int[] rows = this.tblSource.getSelectedRows();
		for (int i : rows) {
			al.add(((FileWrapper)this.tblSource.getValueAt(i, this.tblSource.convertColumnIndexToView(0))).getFile());
		}
		return al;
	}
	
	private ArrayList<File> getAllFiles() {
		ArrayList<File> al = new ArrayList<File>();
		for (int i = 0; i < this.tblSource.getRowCount(); ++i) {
			al.add(((FileWrapper)this.tblSource.getValueAt(i, this.tblSource.convertColumnIndexToView(0))).getFile());
		}
		return al;
	}
	
	private void copyImages(ArrayList<File> alFiles) {
		if (alFiles.size() == 1) {
			JFileChooser fileChooser = new JFileChooser();
			fileChooser.setSelectedFile(new File(alFiles.get(0).getName()));
			int returnCode = fileChooser.showSaveDialog(this.tblSource);
			
			if (returnCode != JFileChooser.APPROVE_OPTION || fileChooser.getSelectedFile().isDirectory()) {
				return;
			}
			if (fileChooser.getSelectedFile().exists()) {
				int del = JOptionPane.showConfirmDialog(this.tblSource, this.lib.getValue("search.ResultScrollPane.fileExists.text"),
						this.lib.getValue("search.ResultScrollPane.fileExists.title"), JOptionPane.YES_NO_OPTION);
				if (del == JOptionPane.NO_OPTION) return;
				fileChooser.getSelectedFile().delete();
			}
			try {
				copyFile(alFiles.get(0), fileChooser.getSelectedFile());
			} catch (IOException e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, e.getLocalizedMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			}
			return;
		}
		
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int returnCode = fileChooser.showSaveDialog(this.tblSource);
		
		if (returnCode != JFileChooser.APPROVE_OPTION || !fileChooser.getSelectedFile().exists() ||
				!fileChooser.getSelectedFile().isDirectory()) {
			return;
		}
		
		File targetDir = fileChooser.getSelectedFile();
		for (File f : alFiles) {
			try {
				File target = new File(targetDir.getAbsolutePath() + File.separator + f.getName());
				if (target.exists()) {
					String sFirstNamePart = f.getName().substring(0, f.getName().lastIndexOf("."));
					String sSecondNamePart = f.getName().substring(f.getName().lastIndexOf("."));
					int i = 1;
					do {
						target = new File(targetDir.getAbsolutePath() + File.separator + sFirstNamePart + "[" + i + "]" + sSecondNamePart);
						++i;
					} while (target.exists());
				}
				copyFile(f, target);
			} catch (IOException e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, e.getLocalizedMessage(), "Error", JOptionPane.ERROR_MESSAGE);
				
				// if some images were allready copied, delete them
				//for (File del : targetDir.listFiles()) {
				//	del.delete();
				//}
				return;
			}
		}
	}
	
	private void copyUrl(ArrayList<File> alFiles) {
		StringSelection stringSelection = new StringSelection(alFiles.get(0).getAbsolutePath());
	    Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
	    clipboard.setContents(stringSelection, new ClipboardOwner() {
			public void lostOwnership(Clipboard arg0, Transferable arg1) {}});
	}
	
	private void copyInfo() {
		int row = this.tblSource.rowAtPoint(this.pMenuSource);
		StringBuilder sClip = new StringBuilder("");
		String sTmp;
		String sNewline = System.getProperty("line.separator");
		
		// address
		sTmp = (((FileWrapper)this.tblSource.getValueAt(row, this.tblSource.convertColumnIndexToView(0))).getFile()).getAbsolutePath();
		sClip.append(this.lib.getValue("search.ResultScrollPane.column.name") + " = " + sTmp);
		sClip.append(sNewline);
		
		// artist
		sTmp = (String)this.tblSource.getValueAt(row, this.tblSource.convertColumnIndexToView(1));
		if (!sTmp.trim().equals("")) {
			sClip.append(this.lib.getValue("search.ResultScrollPane.column.artist") + " = " + sTmp);
			sClip.append(sNewline);
		}
		
		// headcount
		sTmp = "" + this.tblSource.getValueAt(row, this.tblSource.convertColumnIndexToView(2));
		if (!sTmp.trim().equals("0")) {
			sClip.append(this.lib.getValue("search.ResultScrollPane.column.headcount") + " = " + sTmp);
			sClip.append(sNewline);
		}
		
		// spezies
		sTmp = (String)this.tblSource.getValueAt(row, this.tblSource.convertColumnIndexToView(3));
		if (!sTmp.trim().equals("")) {
			sClip.append(this.lib.getValue("search.ResultScrollPane.column.spezies") + " = " + sTmp);
			sClip.append(sNewline);
		}
		
		// gender
		sTmp = (String)this.tblSource.getValueAt(row, this.tblSource.convertColumnIndexToView(4));
		if (!sTmp.trim().equals("")) {
			sClip.append(this.lib.getValue("search.ResultScrollPane.column.gender") + " = " + sTmp);
			sClip.append(sNewline);
		}
		
		// yiff
		sTmp = (String)this.tblSource.getValueAt(row, this.tblSource.convertColumnIndexToView(5));
		if (!sTmp.trim().equals("")) {
			sClip.append(this.lib.getValue("search.ResultScrollPane.column.yiff") + " = " + sTmp);
			sClip.append(sNewline);
		}
		
		// fetish
		sTmp = (String)this.tblSource.getValueAt(row, this.tblSource.convertColumnIndexToView(6));
		if (!sTmp.trim().equals("")) {
			sClip.append(this.lib.getValue("search.ResultScrollPane.column.fetish") + " = " + sTmp);
			sClip.append(sNewline);
		}
		
		// other
		sTmp = (String)this.tblSource.getValueAt(row, this.tblSource.convertColumnIndexToView(7));
		if (!sTmp.trim().equals("")) {
			sClip.append(this.lib.getValue("search.ResultScrollPane.column.other") + " = " + sTmp);
			sClip.append(sNewline);
		}
		
		// resolution
		sTmp = this.tblSource.getValueAt(row, this.tblSource.convertColumnIndexToView(8)).toString();
		sClip.append(this.lib.getValue("search.ResultScrollPane.column.resolution") + " = " + sTmp);
		
		StringSelection stringSelection = new StringSelection(sClip.toString());
	    Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
	    clipboard.setContents(stringSelection, new ClipboardOwner() {
			public void lostOwnership(Clipboard arg0, Transferable arg1) {}});
	}
	
	public static void copyFile (File source, File dest) throws IOException {
	     FileChannel in = null, out = null;
	     try {          
	          in = new FileInputStream(source).getChannel();
	          out = new FileOutputStream(dest).getChannel();
	 
	          long size = in.size();
	          MappedByteBuffer buf = in.map(FileChannel.MapMode.READ_ONLY, 0, size);
	 
	          out.write(buf);
	 
	     } finally {
	          if (in != null) in.close();
	          if (out != null) out.close();
	     }
	}
}

class ResolutionWrapper implements Comparable<ResolutionWrapper> {
	int m_iWidth = 0;
	int m_iHeight = 0;
	long m_nPixels = 0;
	public ResolutionWrapper(int width, int height) {
		this.m_iWidth = width;
		this.m_iHeight = height;
		this.m_nPixels = width * height;
	}
	
	@Override
	public String toString() {
		return this.m_iWidth + " x " + this.m_iHeight;
	}
	
	public int compareTo(ResolutionWrapper o) {
		if (equals(o)) return 0;
		if (o.m_nPixels > this.m_nPixels) return -1;
		return 1;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (int) (this.m_nPixels ^ (this.m_nPixels >>> 32));
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		ResolutionWrapper other = (ResolutionWrapper) obj;
		if (this.m_nPixels != other.m_nPixels) return false;
		return true;
	}
}

class FileWrapper implements Comparable<FileWrapper> {
	File m_file = null;
	public FileWrapper(File f) {
		this.m_file = f;
	}
	
	@Override
	public String toString() {
		return this.m_file.getName();
	}
	
	public File getFile() {
		return this.m_file;
	}
	
	public int compareTo(FileWrapper o) {
		return this.m_file.getName().compareTo(o.m_file.getName());
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((this.m_file == null) ? 0 : this.m_file.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		FileWrapper other = (FileWrapper) obj;
		if (this.m_file == null) {
			if (other.m_file != null) return false;
		} else if (!this.m_file.getName().equals(other.m_file.getName())) {
			return false;
		}
		return true;
	}
}