package search;

import index.FilePanel;
import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import allrounder.DbHandler;
import allrounder.StringLib;
import allrounder.TaggedImage;

public class SearchDialog extends JDialog implements ActionListener {
	private static final long serialVersionUID = 3749818275758597835L;
	StringLib lib = StringLib.getInstance();
	JLabel lblDescription = new JLabel(this.lib.getValue("search.SearchDialog.description"));
	JPanel pnlCriteria = new JPanel();
	JScrollPane jspCriteria = new JScrollPane(this.pnlCriteria);
	JButton btnAddCriterium = new JButton(this.lib.getValue("search.SearchDialog.btnAddCriterium.text"));
	JButton btnSearch = new JButton(this.lib.getValue("search.SearchDialog.btnSearch.text"));
	JLabel lblResults = new JLabel(this.lib.getValue("search.SearchDialog.result"));
	JLabel lblResultCount = new JLabel(" ");
	ResultScrollPane jspResult;
	Vector<SearchCriteriaPanel> vectCriteria = new Vector<SearchCriteriaPanel>();
	
	FilePanel filePanel;
	
	public SearchDialog(FilePanel fp) {
		this.jspResult = new ResultScrollPane(fp);
		
		setTitle(this.lib.getValue("search.SearchDialog.title"));
		//setModal(true);
		setSize(900, 600);
		setLocationRelativeTo(fp.getParent());
		((JPanel)this.getContentPane()).setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		this.pnlCriteria.setLayout(new BoxLayout(this.pnlCriteria, BoxLayout.Y_AXIS));
		this.btnAddCriterium.setToolTipText(this.lib.getValue("search.SearchDialog.btnAddCriterium.tt"));
		this.btnAddCriterium.addActionListener(this);
		this.btnAddCriterium.setActionCommand("ADDCRITERIA");
		this.btnSearch.setToolTipText(this.lib.getValue("search.SearchDialog.btnSearch.tt"));
		this.btnSearch.addActionListener(this);
		this.btnSearch.setActionCommand("SEARCH");
		getRootPane().setDefaultButton(this.btnSearch);
		this.lblResults.setFont(this.lblResults.getFont().deriveFont(17f));
		
		this.vectCriteria.add(new SearchCriteriaPanel(false, this));
		
		//column / row
		double[][] layout = {{this.btnAddCriterium.getPreferredSize().width, TableLayoutConstants.FILL},{this.lblDescription.getPreferredSize().height, 10, this.btnAddCriterium.getPreferredSize().height, 5, TableLayoutConstants.FILL, 10, this.btnSearch.getPreferredSize().height, 20, this.lblResults.getPreferredSize().height, 5, TableLayoutConstants.FILL, 5, this.lblResultCount.getPreferredSize().height}};
		this.setLayout(new TableLayout(layout));
		
		this.add(this.lblDescription, "0, 0, 1, 0");
		this.add(this.btnAddCriterium, "0, 2");
		this.add(this.jspCriteria, "0, 4, 1, 4");
		this.add(this.btnSearch, "0, 6");
		this.add(this.lblResults, "0, 8");
		this.add(this.jspResult, "0, 10, 1, 10");
		this.add(this.lblResultCount, "0, 12");
		buildCriteriaPanel();
		setVisible(true);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	}

	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equalsIgnoreCase("ADDCRITERIA")) {
			this.vectCriteria.add(new SearchCriteriaPanel(this));
			buildCriteriaPanel();
		} else if (arg0.getActionCommand().equalsIgnoreCase("CLOSE")) {
			this.vectCriteria.remove(arg0.getSource());
			buildCriteriaPanel();
		} else if (arg0.getActionCommand().equalsIgnoreCase("SEARCH")) {
			setCursor(new Cursor(Cursor.WAIT_CURSOR));
			
			// get all indexed images
			TreeSet<TaggedImage> imgs = DbHandler.getInstance().getAllImages();
			
			for (SearchCriteriaPanel scp : this.vectCriteria) {
				imgs = scp.filterImages(imgs);
			}
			
			this.jspResult.setImages(imgs);
			this.lblResultCount.setText(this.lib.getValue("search.SearchDialog.resultCount", "", " " + imgs.size()));
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
	}
	
	private void buildCriteriaPanel() {
		this.pnlCriteria.removeAll();
		for (SearchCriteriaPanel pnl : this.vectCriteria) {
			this.pnlCriteria.add(pnl);
		}
		this.jspCriteria.revalidate();
		this.jspCriteria.repaint();
	}
}
